#!/usr/bin/env python
#
# This file is part of pacman-mirrors.
#
# pacman-mirrors is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# pacman-mirrors is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with pacman-mirrors.  If not, see <http://www.gnu.org/licenses/>.
#

import argparse


class CustomHelpFormatter(argparse.HelpFormatter):
    """Provide a customized format for help output.
    http://stackoverflow.com/questions/9642692/argparse-help-without-duplicate-allcaps
    https://github.com/Ecogenomics/mingle/blob/master/mingle/custom_help_formatter.py
    """

    def _format_action_invocation(self, action):
        """Removes duplicate ALLCAPS with positional arguments."""
        if not action.option_strings:
            default = self._get_default_metavar_for_positional(action)
            (metavar,) = self._metavar_formatter(action, default)(1)
            return metavar
        else:
            parts = []
            # if the Optional doesn't take a value, format is:
            #    -s, --long
            if action.nargs == 0:
                parts.extend(action.option_strings)
            # if the Optional takes a value, format is:
            #    -s ARGS, --long ARGS
            else:
                default = self._get_default_metavar_for_optional(action)
                args_string = self._format_args(action, default)
                for option_string in action.option_strings:
                    parts.append(option_string)

                return "%s %s" % (", ".join(parts), args_string)
            return ", ".join(parts)



#!/usr/bin/env python

from urllib import request
from urllib import error
import json
from datetime import datetime
from pacman_mirrors.constants import colors
from pacman_mirrors.functions import printFn
from pacman_mirrors.functions.util import strip_protocol
from pacman_mirrors.functions.util import msg
from pacman_mirrors.functions.jsonFn import read_json_file

C_KO = colors.RED
C_OK = colors.GREEN
C_NONE = colors.RESET


def get_local_mirrors() -> tuple:
    urls = []
    try:
        with open("/etc/pacman.d/mirrorlist", "r") as f_list:
            for line in f_list:
                if not line.startswith("Server"):
                    continue
                line = line.split("=")[1].strip()
                line = line.split("$")[0]
                mirror_url = line.split("/")
                mirror_url.pop()
                mirror_branch = mirror_url.pop()
                line = "/".join(mirror_url)
                urls.append(
                    line + "/",
                )

        return mirror_branch, urls

    except (FileNotFoundError, UnboundLocalError):
        return "", []


def get_state(states: list, branch: str) -> tuple:
    ret_color = C_OK
    status_text = "OK"
    x = states[0]
    if branch == "testing":
        x = states[1]

    if branch == "unstable":
        x = states[2]

    if x == 0:
        ret_color = C_KO
        status_text = "--"

    if x == -1:
        ret_color = C_KO
        status_text = "--"

    return ret_color, status_text


def print_status(self) -> int:
    """
    Printe mirror status
    :param self:
    :return:
    """
    # If configuration urls are missing - show only first mirror from mirror pool
    if not self.config["url_mirrors_json"] or not self.config["url_status_json"]:
        color = C_OK
        text = "OK"
        now = datetime.now()
        fake = now.strftime("00:%M")
        mirror = get_static_mirror(self.config["mirror_file"])
        print(
            "Mirror #1",
            color,
            f"{text}",
            C_NONE,
            f"{fake} {mirror['country']} {mirror['url']}",
        )
        return 0

    system_branch, mirrors_pacman = get_local_mirrors()
    # bug out when no local mirrorlist exist or branch is deprecated
    if system_branch == "" or system_branch == "stable-staging":
        print(C_KO, "MIRRORLIST ERROR", C_NONE)
        return 1

    try:
        with request.urlopen("https://repo.manjaro.org/status.json") as f_url:
            req = f_url.read()

    except error.URLError:
        msg("Downloading status failed!", color=colors.BLUE)
        msg("Please check you network connection ...", color=colors.YELLOW)
        return 1

    json_data = json.loads(req)
    mirrors = []
    for mirror in json_data:
        for protocol in mirror["protocols"]:
            temp = mirror.copy()
            temp["url"] = f"{protocol}://{strip_protocol(temp['url'])}"
            mirrors.append(temp)

    mirrors = [m for m in mirrors if m["url"] in mirrors_pacman]
    printFn.yellow_msg(f"Local mirror status for {system_branch} branch")
    exit_code = 0  # up-to-

    for i, url in enumerate(mirrors_pacman):
        # same order as pacman-conf
        try:
            mirror = [m for m in mirrors if m["url"] == url][0]
            color, text = get_state(mirror["branches"], system_branch)
            len_country = max(len(m["country"]) for m in mirrors) + 1
            last_sync = mirror["last_sync"]
            if last_sync == "-1":
                last_sync = "--:--"
                color = C_KO
                text = "--"

            print(
                f"Mirror #{str(i + 1):2}",
                color,
                f"{text}",
                C_NONE,
                f"{last_sync:7} {mirror['country']:{len_country}} {mirror['url']}",
            )

            if i == 0 and color == C_KO:
                # first mirror not sync !
                exit_code = 4

        except IndexError:
            # not found
            print(C_KO, f"Mirror #{i + 1:2}", f"{url} does not exist{C_NONE}")
            exit_code = 5

    return exit_code


def get_static_mirror(filename: str) -> dict:
    """
    Get first mirror from mirror pool
    :param filename:
    :return:
    """
    mirror = read_json_file(filename)
    return mirror[0]



#!/usr/bin/env python
#
# This file is part of pacman-mirrors.
#
# pacman-mirrors is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# pacman-mirrors is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with pacman-mirrors.  If not, see <http://www.gnu.org/licenses/>.
#
# Authors: Esclapion <esclapion@manjaro.org>
#          philm <philm@manjaro.org>
#          Ramon Buldó <rbuldo@gmail.com>
#          Hugo Posnic <huluti@manjaro.org>
#          Frede Hundewadt <echo ZmhAbWFuamFyby5vcmcK | base64 -d>

"""Pacman-Mirrors Main Module"""

import importlib.util
import sys
import logging

from pacman_mirrors.functions import util
from pacman_mirrors.builder import common, fasttrack, interactive
from pacman_mirrors.config import configuration as conf
from pacman_mirrors.functions import cliFn
from pacman_mirrors.functions import config_setup
from pacman_mirrors.functions import defaultFn
from pacman_mirrors.functions import fileFn
from pacman_mirrors.functions import httpFn
from pacman_mirrors.functions import outputFn
from pacman_mirrors.mirrors.mirror import Mirror
from pacman_mirrors.translation import i18n

try:
    importlib.util.find_spec("gi.repository.Gtk")
except ImportError:
    GTK_AVAILABLE = False
else:
    GTK_AVAILABLE = True
_ = i18n.language.gettext


class PacmanMirrors:
    def __init__(self):
        self.arm = False
        self.config = {"config_file": conf.CONFIG_FILE}
        self.continent = False
        self.custom = False
        self.default = False
        self.fasttrack = None
        self.interactive = False
        self.max_wait_time = 2
        self.mirrors = Mirror()
        self.network = True
        self.no_display = False
        self.no_mirrorList = False
        self.no_status = False
        self.quiet = False
        self.selected_countries = []
        self.tty = False
        self.use_timezone = False

    def run(self):
        """
        # Setup config: returns the config dictionary and true/false on custom
        # Check working folders
        # Check for static config
        # Parse commandline
        # Check network
        # aarch64 check
        # sanitize config
        # Update mirror pool
        # Check if mirrorList is not to be touched - normal exit
        # Handle missing network
        # Load default mirror pool
        # Build mirror list
        """
        # load configuration
        (self.config, self.custom) = config_setup.setup_config(self)

        # ensure workdir exist
        fileFn.create_dir(self.config["work_dir"])

        # check for static configuration
        if self.config["static"]:
            # static setup
            self.custom = False
            if fileFn.check_file(self.config["custom_file"]):
                fileFn.delete_file(self.config["custom_file"])

            # write enterprise mirror list
            util.msg("Static configuration found")
            mirrors = util.build_static_from_config(self, self.config["static"])
            outputFn.write_static_mirror_pool(self, mirrors)
            outputFn.write_pacman_mirror_list(self, mirrors)
            exit(0)

        ##########################################################
        # default manjaro setup
        cliFn.parse_command_line(self, gtk_available=GTK_AVAILABLE)

        # check network connection
        self.network = httpFn.check_internet_connection(tty=self.tty)

        # check for arm architecture
        util.aarch64_check(self, write=True)

        # sanitize the configuration
        if not config_setup.sanitize_config(config=self.config):
            sys.exit(2)

        # download mirror pool if internet connection is available
        if self.network:
            httpFn.download_mirror_pool(
                config=self.config, tty=self.tty, quiet=self.quiet
            )

        # conditional exit
        # used in pacman hook to avoid changing mirror pool and mirror list on updates
        if self.no_mirrorList:
            sys.exit(0)

        # conditional reset CLI configuration
        # if internet connection is down
        if not self.network:
            if not self.quiet:
                util.internet_message(tty=self.tty)
            self.config["method"] = "random"
            self.fasttrack = False

        # Load a configured mirror pool
        defaultFn.load_config_mirror_pool(self)

        # Decide which type of mirrorlist to create Fasttrack/Interactive/Default
        if self.fasttrack:
            fasttrack.build_mirror_list(self, limit=self.fasttrack)
        elif self.interactive:
            interactive.build_mirror_list(self)
        else:
            common.build_mirror_list(self)


def start():
    app = PacmanMirrors()
    app.run()


if __name__ == "__main__":
    start()



#!/usr/bin/env python
#
# This file is part of pacman-mirrors.
#
# pacman-mirrors is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# pacman-mirrors is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with pacman-mirrors.  If not, see <http://www.gnu.org/licenses/>.
#
# Authors: Esclapion
#          Hugo Posnic <huluti@manjaro.org>

"""Pacman-Mirrors Color Constants Module"""

RED = "\033[1;31m"
GREEN = "\033[1;32m"
YELLOW = "\033[1;33m"
BLUE = "\033[1;34m"
WHITE = "\033[1;37m"
RESET = "\033[1;m"
DBG_CLR = "\033[1;46m.: {}\033[1;m".format("DEBUG")



#!/usr/bin/env python
#
# This file is part of pacman-mirrors.
#
# pacman-mirrors is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# pacman-mirrors is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with pacman-mirrors.  If not, see <http://www.gnu.org/licenses/>.
#
# Authors: Frede Hundewadt <echo ZmhAbWFuamFyby5vcmcK | base64 -d>

"""Pacman-Mirrors Console UI Module"""

from collections import namedtuple
import npyscreen
from pacman_mirrors.functions import conversion
from pacman_mirrors.constants import txt
from pacman_mirrors.translation import i18n

_ = i18n.language.gettext


class ConsoleUI(npyscreen.NPSAppManaged):
    def __init__(self, server_list: list, random: bool, default: bool):
        npyscreen.NPSAppManaged.__init__(self)
        self.title = txt.I_TITLE_RANDOM if random else txt.I_TITLE
        if default:
            self.title = "Pacman-Mirrors"

        # Server lists
        self.custom_list = []

        self.is_done = False
        self.random = random
        self.default = default

        header_cols = {
            "country": txt.I_COUNTRY,
            "resp_time": txt.I_RESPONSE,
            "last_sync": txt.I_LAST_SYNC,
            "url": txt.I_URL,
        }

        template = namedtuple("Server", ["country", "resp_time", "last_sync", "url"])

        main_server_list = [header_cols]
        main_server_list.extend(server_list)
        servers = conversion.list_to_tuple(
            list_data=main_server_list, named_tuple=template
        )
        self.list_title = txt.I_LIST_TITLE
        self.server_rows = conversion.rows_from_tuple(servers=servers)
        self.header_row = "{:<5}".format(txt.I_USE) + (
            self.server_rows[0].replace("|", " ").strip()
        )

        del self.server_rows[0]

    def main(self):
        # setup form
        mainform = npyscreen.Form(name=self.title)
        mainform.add(npyscreen.TitleFixedText, name=self.list_title)
        mainform.add(npyscreen.TitleFixedText, name=self.header_row)
        selected_servers = mainform.add(
            npyscreen.MultiSelect,
            max_height=0,
            name=self.list_title,
            value=[],
            values=self.server_rows,
            scroll_exit=True,
        )

        mainform.edit()  # activate form
        self.done(selection=selected_servers.get_selected_objects())

    def done(self, selection: list) -> None:
        """After editing
        :param selection:
        """
        if selection:
            for mirror in selection:
                server = mirror.split("|")
                self.custom_list.append(
                    {
                        "country": server[0].strip(),
                        "resp_time": server[1].strip(),
                        "last_sync": server[2].strip(),
                        "url": server[3].strip(),
                    }
                )

        self.is_done = True
        self.setNextForm(None)


def run(server_list: list, random: bool, default: bool) -> object:
    app = ConsoleUI(server_list=server_list, random=random, default=default)

    app.run()
    return app



#!/usr/bin/env python
#
# This file is part of pacman-mirrors.
#
# pacman-mirrors is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# pacman-mirrors is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with pacman-mirrors.  If not, see <http://www.gnu.org/licenses/>.
#
# Authors: Frede Hundewadt <echo ZmhAbWFuamFyby5vcmcK | base64 -d>

"""Pacman-Mirrors Output Functions"""

from pacman_mirrors.constants import txt
from pacman_mirrors.functions import customFn
from pacman_mirrors.functions import defaultFn
from pacman_mirrors.functions import fileFn
from pacman_mirrors.functions import jsonFn
from pacman_mirrors.functions import util


def write_static_mirror_pool(self, selected_mirrors: list) -> None:
    """
    Output enterprise setup to mirror file
    :param self:
    :param selected_mirrors:
    :return:
    """
    jsonFn.write_json_file(data=selected_mirrors, filename=self.config["mirror_file"])


def write_custom_mirrors_json(self, selected_mirrors: list) -> None:
    """
    Output selected mirrors to custom mirror file
    :param self:
    :param selected_mirrors:
    :return:
    """
    util.msg(message=f"{txt.CUSTOM_MIRROR_LIST}", urgency=txt.INF_CLR, tty=self.tty)
    util.msg(
        message="------------------------------------------------------------",
        tty=self.tty,
    )
    jsonFn.write_json_file(data=selected_mirrors, filename=self.config["custom_file"])
    util.msg(
        message=f'{txt.CUSTOM_MIRROR_FILE_SAVED}: {self.config["custom_file"]}',
        urgency=txt.INF_CLR,
        tty=self.tty,
    )


def write_pacman_mirror_list(self, selected_servers) -> None:
    """
    Outputs selected servers to mirrorlist
    :param self:
    :param selected_servers:
    """
    if self.custom:
        fileFn.write_mirror_list(
            config=self.config,
            servers=selected_servers,
            tty=self.tty,
            custom=self.custom,
            quiet=self.quiet,
            interactive=True,
        )
    else:
        fileFn.write_mirror_list(
            config=self.config, servers=selected_servers, tty=self.tty, quiet=self.quiet
        )


def tty_custom_pool(self) -> None:
    """
    List available countries from custom pool
    """
    customFn.load_custom_pool(self)
    print("{}".format("\n".join(self.mirrors.country_pool)))


def tty_default_pool(self) -> None:
    """
    List all available countries
    """
    defaultFn.load_default_mirror_pool(self)
    print("{}".format("\n".join(self.mirrors.country_pool)))



#!/usr/bin/env python
#
# This file is part of pacman-mirrors.
#
# pacman-mirrors is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# pacman-mirrors is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with pacman-mirrors.  If not, see <http://www.gnu.org/licenses/>.
#
# Authors: Frede Hundewadt <echo ZmhAbWFuamFyby5vcmcK | base64 -d>

"""Pacman-Mirrors Test Mirror Functions"""

import sys
from operator import itemgetter

from pacman_mirrors.constants import txt, colors as color
from pacman_mirrors.functions import util
from pacman_mirrors.functions.httpFn import get_mirror_response


def test_mirror_pool(self, worklist: list, limit=None) -> list:
    """
    Query server for response time
    """
    if self.custom:
        util.msg(
            message=f"{txt.USING_CUSTOM_FILE}", urgency=f"{txt.INF_CLR}", tty=self.tty
        )
    else:
        util.msg(
            message=f"{txt.USING_DEFAULT_FILE}", urgency=f"{txt.INF_CLR}", tty=self.tty
        )

    util.msg(
        message=f"{txt.QUERY_MIRRORS} - {txt.TAKES_TIME}",
        urgency=f"{txt.INF_CLR}",
        tty=self.tty,
    )

    counter = 0
    cols, lines = util.terminal_size()
    # set connection timeouts
    http_wait = self.max_wait_time
    ssl_wait = self.max_wait_time * 2
    ssl_verify = self.config["ssl_verify"]
    result = []

    for mirror in worklist:
        # get a list of mirror dictionaries ordered by protocol descending
        # mirror_protocols is monkey patched to return only the first
        # priority:
        #      - https
        #      - http
        #      - ftp
        work_mirror = mirror_protocols(mirror)
        colon = work_mirror[0]["url"].find(":")
        url = work_mirror[0]["url"][colon:]

        for mirror_proto in work_mirror:
            # get protocol
            proto = mirror_proto["protocols"][0]
            # generate url with protocol
            mirror_proto["url"] = f"{proto}{url}"
            # create message for later display
            message = f'  ..... {mirror_proto["country"]:<15}: {mirror_proto["url"]}'

            # if self.tty do not print theis
            if not self.quiet:
                if self.tty:
                    pass
                else:
                    print("{:.{}}".format(message, cols), end="")
                    sys.stdout.flush()

            # https/ftps sometimes takes longer for handshake
            if proto.endswith("tps"):  # https or ftps
                self.max_wait_time = ssl_wait
            else:
                self.max_wait_time = http_wait

            # let's see how responsive you are
            mirror_proto["resp_time"] = get_mirror_response(
                url=mirror_proto["url"],
                config=self.config,
                tty=self.tty,
                timeout=self.max_wait_time,
                quiet=self.quiet,
                ssl_verify=ssl_verify,
            )

            # create a printable string version from the response with appended zeroes
            r_str = str(mirror_proto["resp_time"])
            while len(r_str) < 5:
                r_str += "0"

            # validate against the defined wait time
            if mirror_proto["resp_time"] >= self.max_wait_time:
                # skip line - but not if tty
                if not self.quiet:
                    if self.tty:
                        pass
                    else:
                        print("\r")
            else:
                # only print if not tty
                if not self.quiet:
                    if self.tty:
                        pass
                    else:
                        print(f"\r  {color.GREEN}{r_str}{color.RESET}")

            # we have tty then we print with response time
            if self.tty:
                util.msg(message=message.replace(".....", r_str), tty=self.tty)
                sys.stdout.flush()

        probed_mirror = filter_bad_http(work=work_mirror)

        if limit is not None:
            if mirror["resp_time"] == txt.SERVER_RES:
                continue

            counter += 1
            result.append(probed_mirror)
        else:
            result.append(probed_mirror)
        # Equality check will stop execution
        # when the desired number is reached.
        # In the possible event the first mirror's
        # response time exceeds the predefined response time,
        # the loop would stop execution if the check for zero is not present
        if limit is not None and counter != 0 and counter == limit:
            break

    return result


def mirror_protocols(mirror: dict) -> list:
    """
    Return a number of copies of mirror - one copy per protocol
    :param: mirror dictionary with a number of protocols
    :return: list of mirror dictionaries with one protocol per dictionary
    """
    result = []
    for idx, protocol in enumerate(mirror["protocols"]):
        m = {
            "branches": mirror["branches"],
            "country": mirror["country"],
            "last_sync": mirror["last_sync"],
            "protocols": [protocol],
            "resp_time": mirror["resp_time"],
            "url": mirror["url"],
        }
        result.append(m)
    # -------------------------------------------------
    # monkey patch - sort by protocol desc
    # return the first to avoid testing all protocols for one mirror
    result = sorted(result, key=itemgetter("protocols"), reverse=True)
    return [result[0]]


def filter_bad_http(work: list) -> dict:
    """
    filter bad http/ssl if mirror has more than one protocol
    :param work: list of mirror dictionaries with one protocol per dictionary
    :return: mirror dictionary with invalid ssl removed
    """
    result = {
        "branches": work[0]["branches"],
        "country": work[0]["country"],
        "last_sync": work[0]["last_sync"],
        "protocols": [],
        "resp_time": "",
        "url": work[0]["url"],
    }

    if len(work) > 1:
        for item in work:
            if (
                item["protocols"][0].endswith("tps")
                and item["resp_time"] == txt.SERVER_RES
            ):
                continue
            result["protocols"].append(item["protocols"][0])
            result["resp_time"] = item["resp_time"]

        return result

    return work[0]



#!/usr/bin/env python
#
# This file is part of pacman-mirrors.
#
# pacman-mirrors is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# pacman-mirrors is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with pacman-mirrors.  If not, see <http://www.gnu.org/licenses/>.
#
# Authors: Frede Hundewadt <echo ZmhAbWFuamFyby5vcmcK | base64 -d>

"""Pacman-Mirrors Custom Pool/Mirror Functions"""

from pacman_mirrors.functions import defaultFn
from pacman_mirrors.functions import fileFn
from pacman_mirrors.functions import jsonFn
from pacman_mirrors.functions import validFn


def apply_status_to_custom_pool(config: dict, custom_pool: list) -> list:
    """
    Apply the current mirror status to the custom mirror file
    :param config: config dictionary
    :param custom_pool: the custom mirror pool
    :return: custom mirror pool with status applied
    """
    status_list = tuple(
        jsonFn.read_json_file(filename=config["status_file"], use_ordered_dict=False)
    )
    custom_list = tuple(custom_pool)
    try:
        _ = status_list[0]
        for custom in custom_list:
            for status in status_list:
                if custom["url"] in status["url"]:
                    custom["last_sync"] = status["last_sync"]
                    custom["branches"] = status["branches"]

        return list(custom_list)

    except (IndexError, KeyError):
        return custom_pool


def check_custom_pool(self) -> bool:
    """
    pool or countries from CLI
    """
    self.custom = validFn.custom_config_is_valid()
    if not self.custom:
        self.selected_countries = self.config["country_pool"]

    return self.custom


def delete_custom_pool(self) -> None:
    self.custom = False
    self.config["country_pool"] = []
    fileFn.delete_file(self.config["custom_file"])


def load_custom_pool(self) -> None:
    if self.default:
        defaultFn.load_default_mirror_pool(self)
    else:
        defaultFn.seed_mirrors(self, self.config["custom_file"])
        self.mirrors.mirror_pool = apply_status_to_custom_pool(
            config=self.config, custom_pool=self.mirrors.mirror_pool
        )



#!/usr/bin/env python
#
# This file is part of pacman-mirrors.
#
# pacman-mirrors is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# pacman-mirrors is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with pacman-mirrors.  If not, see <http://www.gnu.org/licenses/>.
#
# Authors: Frede Hundewadt <echo ZmhAbWFuamFyby5vcmcK | base64 -d>

"""Pacman-Mirrors Interactive Fasttrack Mirror List Builder Module"""

from random import shuffle

from pacman_mirrors.builder.builder import build_pool
from pacman_mirrors.constants import txt
from pacman_mirrors.functions import util
from pacman_mirrors.functions.outputFn import write_pacman_mirror_list
from pacman_mirrors.functions.sortMirrorFn import sort_mirror_pool
from pacman_mirrors.functions.testMirrorFn import test_mirror_pool


def build_mirror_list(self, limit: int) -> None:
    """
    Build and update the mirror list by probing and sorting available mirrors.

    Attributes
    ----------
    None

    Parameters
    ----------
    limit : int
        The maximum number of mirrors to be processed. If the value is less than or equal
        to 0, or exceeds the total number of available mirrors, the entire list is used.

    Raises
    ------
    IndexError
        Raised if work_pool is empty during mirror processing, which could occur if no
        mirrors pass the test stage.
    """
    work_pool = build_pool(self)
    shuffle(work_pool)
    # probe the mirrors
    if limit <= 0 or limit > len(work_pool):
        limit = len(work_pool)

    work_pool = test_mirror_pool(self=self, worklist=work_pool, limit=limit)
    # Write mirrorlist
    try:
        _ = work_pool[0]
        work_pool = sort_mirror_pool(
            worklist=work_pool, field="resp_time", reverse=False
        )

        write_pacman_mirror_list(self=self, selected_servers=work_pool)
    except IndexError:
        util.msg(message=f"{txt.NO_SELECTION}", urgency=txt.WRN_CLR, tty=self.tty)
        util.msg(message=f"{txt.NO_CHANGE}", urgency=txt.INF_CLR, tty=self.tty)



#!/usr/bin/env python
#
# This file is part of pacman-mirrors.
#
# pacman-mirrors is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# pacman-mirrors is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with pacman-mirrors.  If not, see <http://www.gnu.org/licenses/>.
#
# Authors: Frede Hundewadt <echo ZmhAbWFuamFyby5vcmcK | base64 -d>

"""Pacman-Mirrors Converter Functions"""
import json
from pacman_mirrors.constants import txt
from pacman_mirrors.functions import util
from pacman_mirrors.functions.util import get_protocol_from_url, location_from_url


def translate_interactive_to_pool(selection: list, mirror_pool: list) -> tuple:
    """
    Translate the interactive selection back to mirror pool
    :param selection: the custom mirror selection
    :param mirror_pool: the default mirror pool
    :return: tuple with custom mirror pool and mirrors for mirror list generation
    """
    custom_pool = []
    mirror_list = []

    for mirror in mirror_pool:
        location = location_from_url(mirror["url"])
        selected = (x for x in selection if location in x["url"])
        protocols = []
        for item in selected:
            protocols.append(get_protocol_from_url(item["url"]))
        if protocols:
            m = mirror
            m["protocols"] = protocols
            mirror_list.append(m)
            custom_pool.append(mirror)
    return custom_pool, mirror_list


def translate_pool_to_interactive(mirror_pool: list, tty: bool = False) -> list:
    """
    Translate mirror pool for interactive display
    :param tty:
    :param mirror_pool:
    :return: list of dictionaries
            {
                "country": "country_name",
                "resp_time": "m.sss",
                "last_sync": "HHh MMm",
                "url": "http://server/repo/"
            }
    """
    interactive_list = []
    for mirror in mirror_pool:
        try:
            _ = mirror_pool[0]
            last_sync = str(mirror["last_sync"]).split(":")
            mirror_url = location_from_url(mirror["url"])
            for idx, protocol in enumerate(mirror["protocols"]):
                interactive_list.append(
                    {
                        "country": mirror["country"],
                        "resp_time": str(mirror["resp_time"]),
                        "last_sync": f"{last_sync[0]}h {last_sync[1]}m",
                        "url": f"{protocol}{mirror_url}",
                    }
                )
        except (KeyError, IndexError):
            util.msg(
                message=f"{txt.HOUSTON}! {txt.MIRROR_POOL_EMPTY}!",
                urgency=txt.WRN_CLR,
                tty=tty,
            )
            break
    return interactive_list


def list_to_tuple(list_data: list, named_tuple) -> list:
    """Comvert list to a list with named tuples
    :param list_data: the list to convert
    :param named_tuple: tuple list item converts to
    :return data: list of named tuples
    """
    tdata = json.dumps(list_data)
    data = json.loads(tdata, object_hook=lambda x: named_tuple(**x))
    return data


def rows_from_tuple(servers: list, join_string: str = " | ") -> list:
    """Generates equal formatted lines
    :param servers: named tuples
    :param join_string: string used to join tuple items
    :return lines: list of nicely formatted lines
    """
    rows = []
    if servers:
        col_width = [max(len(text) for text in col) for col in zip(*servers)]
        for line in servers:
            rows.append(
                join_string.join(
                    "{:{}}".format(text, col_width[i]) for i, text in enumerate(line)
                )
            )
    return rows



#!/usr/bin/env python
#
# This file is part of pacman-mirrors.
#
# pacman-mirrors is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# pacman-mirrors is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with pacman-mirrors.  If not, see <http://www.gnu.org/licenses/>.
#
# Authors: Frede Hundewadt <echo ZmhAbWFuamFyby5vcmcK | base64 -d>

"""Pacman-Mirrors Utility Functions"""

import platform
import shutil
from pacman_mirrors.api import apifn
from pacman_mirrors.constants import txt
from pacman_mirrors.functions import httpFn


def get_zone_countries() -> list:
    """
    Retrieves a list of countries with their associated timezones and related details.

    This function processes and collects country information including timezones,
    continent, region, country name, and country code. Valid timezones are
    filtered for each country, and the information is organized into a dictionary
    format. The result is a list containing these dictionaries for all available
    countries.

    Returns:
        list: A list of dictionaries where each dictionary contains the following:
            - timezones (list): A list of timezone strings associated with a country.
            - continent (str): The continent derived from the first timezone of the country.
            - region (str): The region derived from the first timezone of the country.
            - name (str): The human-readable name of the country.
            - code (str): The ISO 3166-1 alpha-2 country code.

    """
    from zoneinfo import ZoneInfo
    from pytz import country_names, country_timezones

    countries = []
    for country_code in country_timezones:
        # Filter timezones for allowed regions
        filtered_zones = [zone for zone in country_timezones[country_code]]

        if filtered_zones:  # Only process if there are valid timezones
            pm_timezone = {
                "timezones": filtered_zones,
                "continent": ZoneInfo(filtered_zones[0]).key.split("/")[0],
                "region": ZoneInfo(filtered_zones[0]).key.split("/")[0],
                "name": country_names[country_code],
                "code": country_code,
            }
            countries.append(pm_timezone)
    return countries


def extract_mirror_url(data: str) -> str:
    """Extract mirror url from data"""
    line = data.strip()
    if line.startswith("Server"):
        return line[9:].replace("$branch/$repo/$arch", "")
    return ""


def get_country(data: str) -> str:
    """Extract mirror country from data"""
    line = data.strip()
    if line.startswith("[") and line.endswith("]"):
        return line[1:-1]

    elif line.startswith("## Country") or line.startswith("## Location"):
        return line[19:]
    return ""


def get_protocol(url: str) -> str:
    """Extract protocol from url"""
    pos = url.find(":")
    return url[:pos]


def get_protocol_from_url(url: str) -> str:
    """
    Splits an url
    :param url:
    :returns protocol eg. http
    """
    colon = url.find(":")
    if colon:
        return url[:colon]

    return url


def location_from_url(url: str) -> str:
    """
    Splits an url
    :param url:
    :returns url string without protocol
    """
    colon = url.find(":")
    if colon:
        return url[colon:]

    return url


def aarch64_check(self, write: bool = False) -> None:
    if platform.machine() == "aarch64":
        self.config["arm"] = True
        # get branch from config object
        branch = self.config["branch"]
        # branch should start with 'arm-'
        if not branch.startswith("arm-"):
            # change the internal configuration
            # branch to arm-{stable,testing,unstable}
            self.config["branch"] = f"arm-{branch}"
            if write:
                # write the branch to config storage
                apifn.write_config_branch(
                    self.config["branch"], self.config["config_file"], quiet=True
                )


def internet_message(tty: bool = False) -> None:
    """Message when internet connection is down"""
    msg(f"{txt.INTERNET_DOWN}", urgency=txt.INF_CLR, tty=tty)
    msg(f"{txt.MIRROR_RANKING_NA}", urgency=txt.INF_CLR, tty=tty)
    msg(f"{txt.INTERNET_ALTERNATIVE}", urgency=txt.INF_CLR, tty=tty)


def msg(
    message: str,
    urgency: str = "",
    tty: bool = False,
    color: str = "",
    newline: bool = False,
) -> None:
    """Helper for printing messages
    :param message:
    :param urgency:
    :param tty:
    :param color:
    :param newline:
    """
    reset = "\033[1;m"
    if urgency and color:
        color = ""
    if tty:
        if newline:
            print("\n")

        print(f"::{message}")

    else:
        if urgency:
            if newline:
                print("\n")

            print(f"::{urgency} {message}")

        else:
            if newline:
                print("\n")

            print(f"::{color}{message}{reset}")


def strip_protocol(url: str) -> str:
    return url.split("//")[1]


def terminal_size() -> tuple:
    """get terminal size"""
    # http://www.programcreek.com/python/example/85471/shutil.get_terminal_size
    cols = shutil.get_terminal_size().columns
    lines = shutil.get_terminal_size().lines
    result = (cols, lines)
    return result


def build_static_from_config(self, static: str) -> list:
    urls = static.split(",")
    result = []
    for url in urls:
        if not str(url).endswith("/"):
            url = f"{url}/"
        temp = {
            "branches": [1, 1, 1],
            "country": "Static",
            "protocols": [str(url).split(":")[0]],
            "last_sync": "00:05",
            "resp_time": httpFn.get_mirror_response(url, self.config, self.tty),
            "url": url,
        }
        result.append(temp)
    return result



#!/usr/bin/env python
#
# This file is part of pacman-mirrors.
#
# pacman-mirrors is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# pacman-mirrors is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with pacman-mirrors.  If not, see <http://www.gnu.org/licenses/>.
#
# Authors: Frede Hundewadt <echo ZmhAbWFuamFyby5vcmcK | base64 -d>

"""Pacman-Mirrors Command Line Parser"""

# TODO add missing translations to help section

import argparse
import os
import sys
from pacman_mirrors import __version__
from pacman_mirrors.api import api_handler
from pacman_mirrors.functions import customFn
from pacman_mirrors.constants import txt
from pacman_mirrors.functions import outputFn
from pacman_mirrors.functions import mirror_status
from pacman_mirrors.functions import printFn
from pacman_mirrors.translation.custom_help_formatter import CustomHelpFormatter


def parse_command_line(self, gtk_available: bool) -> None:
    """Read the arguments of the command line"""

    args_summary = (
        f"[-h] [-f [{txt.NUMBER}]] [-i [-d]] [-m {txt.METHOD}] [--status]\n"
        f"\t\t[-c {txt.COUNTRY} [{txt.COUNTRY}...] | [--timezone] | [--continent]]\n"
        f"\t\t[-l] [-lc] [-q] [-t {txt.SECONDS}] [-v] [-n]\n"
        f"\t\t[--api] [-S/-B {txt.BRANCH}] [-p {txt.PREFIX}]\n"
        f"\t\t\t[-P {txt.PROTO} [{txt.PROTO}...]] [-R] [-U {txt.URL}]"
    )

    usage_header = f"\rVersion {__version__}\n{txt.USAGE}:\n pacman-mirrors"
    usage = f"{usage_header} {args_summary}"

    parser = argparse.ArgumentParser(
        formatter_class=CustomHelpFormatter, add_help=False, usage=usage
    )

    # Method arguments
    methods = parser.add_argument_group(txt.METHODS)
    methods.add_argument(
        "-i", "--interactive", action="store_true", help=txt.HLP_ARG_INTERACTIVE
    )

    methods_exclusive = methods.add_mutually_exclusive_group()
    methods_exclusive.add_argument(
        "-f",
        "--fasttrack",
        nargs="?",
        const=-1,
        type=int,
        metavar=txt.NUMBER,
        help=f"{txt.HLP_ARG_FASTTRACK} {txt.OVERRIDE_OPT}",
    )

    methods_exclusive.add_argument(
        "-c",
        "--country",
        type=str,
        nargs="+",
        metavar=txt.COUNTRY,
        help=txt.HLP_ARG_COUNTRY,
    )

    methods_exclusive.add_argument(
        "-z", "--timezone", action="store_true", help=txt.HLP_ARG_TIMEZONE
    )

    methods_exclusive.add_argument(
        "--continent", action="store_true", help=txt.HLP_ARG_CONTINENT
    )

    api = parser.add_argument_group(txt.API)
    api.add_argument(
        "-a",
        "--api",
        action="store_true",
        help=f"[-p {txt.PREFIX}][-R][-S/-B|-G {txt.BRANCH}][-P {txt.PROTO} [{txt.PROTO} ...]]",
    )

    api.add_argument(
        "-S",
        "-B",
        "--set-branch",
        choices=["stable", "testing", "unstable"],
        help=f"{txt.API}: {txt.HLP_ARG_API_SET_BRANCH}",
    )

    api.add_argument(
        "-p",
        "--prefix",
        type=str,
        metavar=txt.PREFIX,
        help=f"{txt.API}: {txt.HLP_ARG_API_PREFIX} {txt.PREFIX_TIP}",
    )

    api.add_argument(
        "-P",
        "--proto",
        "--protocols",
        choices=["all", "http", "https", "ftp", "ftps"],
        type=str,
        nargs="+",
        help=f"{txt.API}: {txt.HLP_ARG_API_PROTOCOLS}",
    )

    api.add_argument(
        "-R",
        "--re-branch",
        action="store_true",
        help=f"{txt.API}: {txt.HLP_ARG_API_RE_BRANCH}",
    )

    api.add_argument(
        "-U",
        "--url",
        type=str,
        metavar=txt.URL,
        help=f"{txt.API}: {txt.HLP_ARG_API_URL}",
    )

    misc = parser.add_argument_group(txt.MISC)
    misc.add_argument(
        "-G", "--get-branch", action="store_true", help=txt.HLP_ARG_API_GET_BRANCH
    )

    misc.add_argument(
        "-d",
        "--default",
        action="store_true",
        help=f"INTERACTIVE: {txt.HLP_ARG_DEFAULT}",
    )

    misc.add_argument("-h", "--help", action="store_true")
    misc.add_argument(
        "-l", "--list", "--country-list", action="store_true", help=txt.HLP_ARG_LIST
    )

    misc.add_argument(
        "-lc",
        "--country-config",
        action="store_true",
        help=txt.HLP_ARG_LIST_COUNTRIES,
    )

    misc.add_argument(
        "-m", "--method", type=str, choices=["rank", "random"], help=txt.HLP_ARG_METHOD
    )

    misc.add_argument(
        "-n", "--no-mirrorlist", action="store_true", help=txt.HLP_ARG_NO_MIRRORLIST
    )

    misc.add_argument("-q", "--quiet", action="store_true", help=txt.HLP_ARG_QUIET)

    misc.add_argument("-s", "--no-status", action="store_true", help=txt.HLP_ARG_STATUS)

    misc.add_argument(
        "-t", "--timeout", type=int, metavar=txt.SECONDS, help=txt.HLP_ARG_TIMEOUT
    )

    misc.add_argument("-v", "--version", action="store_true", help=txt.HLP_ARG_VERSION)

    misc.add_argument("--no-color", action="store_true")

    misc.add_argument("--interval", type=int, help=txt.HLP_ARG_INTERVAL)

    misc.add_argument("-g", action="store_true", help=txt.HLP_ARG_GENERATE)

    misc.add_argument("--status", action="store_true", help=txt.HLP_ARG_MIRROR_STATUS)

    misc.add_argument("-fc", action="store_true")

    args = parser.parse_args()

    # No root required
    if len(sys.argv) == 1:
        printFn.blue_msg(f"Pacman-mirrors version {__version__}")
        exit_code = mirror_status.print_status(self)
        sys.exit(exit_code)

    if args.help:
        parser.print_help()
        sys.exit(0)

    if args.version:
        print(__version__)
        sys.exit(0)

    if args.list:
        outputFn.tty_default_pool(self)
        sys.exit(0)

    if args.country_config:
        outputFn.tty_custom_pool(self)
        sys.exit(0)

    if args.get_branch:
        print(self.config["branch"])
        sys.exit(0)

    if args.no_color:
        self.tty = args.no_color

    if args.status:
        self.status = args.status
        exit_code = mirror_status.print_status(self)
        sys.exit(exit_code)

    # Validate arg combinations
    # If --set-branch, --protocols, --url, --prefix and not --api reject
    if args.set_branch or args.proto or args.url or args.prefix:
        if not args.api:
            print(f"Version {__version__}\nUSAGE:\n pacman-mirrors {args_summary}")
            print("{}".format(txt.API_ARGUMENTS_ERROR))
            sys.exit(1)

    # If --default and not --interactive reject
    if args.default:
        if not args.interactive:
            print(f"Version {__version__}\nUSAGE:\n pacman-mirrors {args_summary}")
            print(txt.INTERACTIVE_ARGUMENTS_ERROR)
            sys.exit(1)

    # If --interval and not --no-status reject
    if args.interval and not args.no_status:
        print(f"Version {__version__}\nUSAGE:\n pacman-mirrors {args_summary}")
        print("Invalid arguments: `--interval` argument only valid with `-s`")
        sys.exit(1)

    # Root required
    if os.getuid() != 0:
        print(f".: {txt.ERR_CLR} {txt.MUST_BE_ROOT}")
        sys.exit(1)

    if args.continent:
        self.continent = True

    if args.timezone:
        self.use_timezone = True

    if args.method:
        self.config["method"] = args.method

    if args.no_mirrorlist:
        self.no_mirrorList = True

    if args.quiet:
        self.quiet = True

    if args.timeout:
        self.max_wait_time = args.timeout
        self.config["timeout"] = args.timeout

    if args.no_status:
        self.no_status = True

    if args.interval:
        self.interval = args.interval
    else:
        self.interval = None

    # Generation methods
    if args.country:
        self.use_timezone = False
        if "," in args.country[0]:
            self.config["country_pool"] = args.country[0].split(",")
        else:
            self.config["country_pool"] = args.country

        if self.config["country_pool"] == ["all"]:
            customFn.delete_custom_pool(self)

    if args.fasttrack is not None:
        self.use_timezone = False
        if args.no_status:
            print(f"Version {__version__}\nUSAGE:\n pacman-mirrors {args_summary}")
            print(f"{txt.FASTTRACK_ARGUMENTS_ERROR}")
            sys.exit(1)

        self.fasttrack = args.fasttrack
        self.config["method"] = "rank"

    if args.interactive:
        self.interactive = True
        if args.default:
            self.default = True

        # wayland - sway - console - gtk
        if (
            os.environ.get("XDG_SESSION_TYPE") == "wayland"
            or os.environ.get("XDG_SESSION_TYPE") == "tty"
            or not os.environ.get("DISPLAY")
            or not gtk_available
        ):
            self.no_display = True

    if args.fc:
        self.no_display = True

    # API handling
    # Setup variables for passing to the api_config function
    if args.api:
        rebranch = False
        url = args.url
        setbranch = args.set_branch
        setprotocols = bool(args.proto)

        if args.re_branch:
            rebranch = True
        if args.proto:
            if "all" in args.proto:
                self.config["protocols"] = []
            else:
                if "," in args.proto:
                    self.config["protocols"] = args.proto.split(",")
                else:
                    self.config["protocols"] = args.proto

        api_handler.set_config(
            self,
            set_pfx=args.prefix,
            set_branch=setbranch,
            re_branch=rebranch,
            set_protocols=setprotocols,
            set_url=url,
        )



#!/usr/bin/env python
#
# This file is part of pacman-mirrors.
#
# pacman-mirrors is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# pacman-mirrors is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with pacman-mirrors.  If not, see <http://www.gnu.org/licenses/>.
#
# Authors: Frede Hundewadt <echo ZmhAbWFuamFyby5vcmcK | base64 -d>

"""Pacman-Mirrors Validation Functions"""

import os
import sys

from pacman_mirrors.config import configuration as conf
from pacman_mirrors.constants import txt
from pacman_mirrors.functions import util


def custom_config_is_valid() -> bool:
    """Check validity of custom selection
    :return: True or False
    :rtype: bool
    """
    return os.path.isfile(conf.CUSTOM_FILE)


def country_list_is_valid(
    onlycountry: list, countrylist: list, tty: bool = False
) -> bool:
    """Check if the list of countries are valid.
    :param onlycountry: list of countries to check
    :param countrylist: list of available countries
    :param tty:
    :return: True
    :rtype: bool
    """
    for country in onlycountry:
        if country not in countrylist:
            util.msg(
                message=f"{txt.OPTION}{txt.OPT_COUNTRY}: {txt.UNKNOWN_COUNTRY} '{country}'",
                urgency=txt.WRN_CLR,
                tty=tty,
            )

            util.msg(message=f"{txt.AVAILABLE_COUNTRIES}", urgency=txt.INF_CLR, tty=tty)

            print(f'{", ".join(countrylist)}')
            sys.exit(1)

    return True



#!/usr/bin/env python
#
# This file is part of pacman-mirrors.
#
# pacman-mirrors is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# pacman-mirrors is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with pacman-mirrors.  If not, see <http://www.gnu.org/licenses/>.
#
# Authors: Frede Hundewadt <echo ZmhAbWFuamFyby5vcmcK | base64 -d>

"""Pacman-Mirrors Print Functions"""

from pacman_mirrors.constants import colors as color


def blue_msg(message: str) -> None:
    print(f"{color.BLUE}{message}{color.RESET}")


def green_msg(message: str) -> None:
    print(f"{color.GREEN}{message}{color.RESET}")


def red_msg(message: str) -> None:
    print(f"{color.RED}{message}{color.RESET}")


def yellow_msg(message: str) -> None:
    print(f"{color.YELLOW}{message}{color.RESET}")



#!/usr/bin/env python
#
# This file is part of pacman-mirrors.
#
# pacman-mirrors is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# pacman-mirrors is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with pacman-mirrors.  If not, see <http://www.gnu.org/licenses/>.
#
# Authors: Frede Hundewadt <echo ZmhAbWFuamFyby5vcmcK | base64 -d>


"""Pacman-Mirrors Mirror Class Module"""

from decimal import Decimal

from pacman_mirrors.constants import txt
from pacman_mirrors.functions.pools import get_continent


class Mirror:
    def __init__(self):
        self.country_pool = []
        self.mirror_pool = []

    def add(
        self,
        country: str,
        url: str,
        protocols: list,
        branches: list = None,
        last_sync: str = None,
        resp_time: str = None,
    ) -> None:
        """Append mirror
        :param country:
        :param url:
        :param protocols:
        :param branches: optional from status.json
        :param last_sync: optional from status.json
        :param resp_time: optional from status.json
        """
        if branches is None:
            branches = [-1, -1, -1]

        if last_sync is None:
            last_sync = "00:00"

        if resp_time is None:
            resp_time = 0
        else:
            resp_time = Decimal(resp_time)

        if country not in self.country_pool:
            self.country_pool.append(country)

        continent = get_continent(country.replace("_", " "))
        if continent is None:
            continent = "CDN"

        # translate negative integer in status.json
        if last_sync == -1:
            last_sync = txt.SERVER_BAD  # "9999:99"
            resp_time = txt.SERVER_RES  # 99.99

        # sort protocols in reversed order https,http,ftps,ftp
        protocols = sorted(protocols, reverse=True)

        # add to pool
        self.mirror_pool.append(
            {
                "continent": continent,
                "branches": branches,
                "country": country,
                "last_sync": last_sync,
                "protocols": protocols,
                "resp_time": resp_time,
                "url": url,
            }
        )

    def seed(self, servers: list, status: bool = False, custom: bool = False) -> None:
        """
        Seed mirrorlist
        :param servers:
        :param status:
        :param custom:
        """
        if custom:  # clear previous data
            self.country_pool = []
            self.mirror_pool = []

        for server in servers:
            if status:
                self.add(
                    server["country"],
                    server["url"],
                    server["protocols"],
                    server["branches"],
                    server["last_sync"],
                )
            else:
                self.add(server["country"], server["url"], server["protocols"])



#!/usr/bin/env python
#
# This file is part of pacman-mirrors.
#
# pacman-mirrors is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# pacman-mirrors is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with pacman-mirrors.  If not, see <http://www.gnu.org/licenses/>.
#
# Authors: Frede Hundewadt <echo ZmhAbWFuamFyby5vcmcK | base64 -d>

"""Pacman-Mirrors JSON Functions"""

import collections
import json


def json_dump_file(data: list, filename: str) -> None:
    with open(filename, "w") as outfile:
        json.dump(data, outfile)


def read_file_content(file_name: str, binary: bool = False) -> str:
    """
    Reads the content of a file.
    """
    mode = "rb" if binary else "r"
    with open(file_name, mode) as file:
        return file.read()


def read_json_file(filename: str, use_ordered_dict: bool = True) -> list:
    """Reads json data from file."""
    data = []  # Represent an empty JSON array
    try:
        file_content = read_file_content(filename, binary=use_ordered_dict)

        if use_ordered_dict:
            # noinspection PyTypeChecker
            data = json.loads(
                file_content.decode("utf8"),
                object_pairs_hook=collections.OrderedDict,
            )
        else:
            data = json.loads(file_content)
    except OSError:
        pass

    return data


def write_json_file(data: list, filename: str, dictionary: bool = False) -> bool:
    """Writes data to file as json
    :param data
    :param filename:
    :param dictionary:
    """
    try:
        # TODO what was the purpose of dictionary?
        if dictionary:
            with open(filename, "w") as outfile:
                json.dump(data, outfile)
        else:
            with open(filename, "w") as outfile:
                json.dump(data, outfile, sort_keys=True, indent=4)

        return True

    except OSError:
        return False



#!/usr/bin/env python
#
# This file is part of pacman-mirrors.
#
# pacman-mirrors is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# pacman-mirrors is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with pacman-mirrors.  If not, see <http://www.gnu.org/licenses/>.
#
# Authors: Frede Hundewadt <echo ZmhAbWFuamFyby5vcmcK | base64 -d>

"""Pacman-Mirrors Interactive Mirror List Builder Module"""

from operator import itemgetter
from random import shuffle

from pacman_mirrors.builder.builder import build_pool
from pacman_mirrors.constants import txt
from pacman_mirrors.functions import util
from pacman_mirrors.functions.sortMirrorFn import sort_mirror_pool
from pacman_mirrors.functions.testMirrorFn import test_mirror_pool
from pacman_mirrors.functions.conversion import (
    translate_interactive_to_pool,
    translate_pool_to_interactive,
)
from pacman_mirrors.functions.outputFn import (
    write_custom_mirrors_json,
    write_pacman_mirror_list,
)


def build_mirror_list(self) -> None:
    """
    Builds a mirror list for the system based on the provided configuration and user
    interactions. This method involves preparation of a work list, optional ranking
    or shuffling, invoking appropriate user interfaces for interaction, processing
    user selections, and optionally writing the results back to the system's mirror
    configuration.

    Raises:
        IndexError: When working on selections or mirror lists that may be empty.
    """
    worklist = build_pool(self)

    # rank or shuffle the mirrorlist before showing the ui
    if not self.default:
        # if not default run method before selection
        if self.config["method"] == "rank":
            # TODO again duplicated objects
            worklist = test_mirror_pool(self=self, worklist=worklist)
            worklist = sort_mirror_pool(
                worklist=worklist, field="resp_time", reverse=False
            )
        else:
            shuffle(worklist)

    # Create a list for display in ui.
    # The gui and the console ui expect the supplied list
    # to be in the old country dictionary format.
    # {
    #    "country": "country_name",
    #    "resp_time": "m.sss",
    #    "last_sync": "HHh MMm",
    #    "url": "http://server/repo/"
    # }
    interactive_list = translate_pool_to_interactive(mirror_pool=worklist, tty=self.tty)

    # import the correct ui
    if self.no_display or self.config["arm"]:
        # in console mode
        from pacman_mirrors.dialogs import consoleui as ui
    else:
        # gtk mode
        from pacman_mirrors.dialogs import graphicalui as ui

    interactive = ui.run(
        server_list=interactive_list,
        random=self.config["method"] == "random",
        default=self.default,
    )

    # process user choices
    if interactive.is_done:
        # translate interactive list back to our json format
        custom_pool, mirror_list = translate_interactive_to_pool(
            selection=interactive.custom_list, mirror_pool=self.mirrors.mirror_pool
        )

        # Try selected method on the mirrorlist
        try:
            _ = mirror_list[0]
            # using the default runs method after selection
            if self.default:
                if self.config["method"] == "rank":
                    # TODO duplicate objects
                    mirror_list = test_mirror_pool(self=self, worklist=mirror_list)
                    mirror_list = sorted(mirror_list, key=itemgetter("resp_time"))
                else:
                    shuffle(mirror_list)
        except IndexError:
            pass

        # Write custom mirror pool
        # Write mirrorlist
        try:
            _ = custom_pool[0]
            self.custom = True
            self.config["country_pool"] = ["Custom"]
            # Writing the custom mirror pool file
            write_custom_mirrors_json(self=self, selected_mirrors=custom_pool)
            # Writing mirrorlist
            # If the mirror list is empty because
            # no up-to-date mirrors exist for user branch
            try:
                # TODO duplicate objects
                _ = mirror_list[0]
                write_pacman_mirror_list(self, mirror_list)
                if self.no_status:
                    util.msg(
                        message=f"{txt.OVERRIDE_STATUS_CHOICE}",
                        urgency=txt.WRN_CLR,
                        tty=self.tty,
                    )
                    util.msg(
                        message=f"{txt.OVERRIDE_STATUS_MIRROR}",
                        urgency=txt.WRN_CLR,
                        tty=self.tty,
                    )
            except IndexError as e:
                util.msg(message=str(e), urgency=txt.WRN_CLR, tty=self.tty)
        except IndexError:
            util.msg(message=f"{txt.NO_SELECTION}", urgency=txt.WRN_CLR, tty=self.tty)
            util.msg(message=f"{txt.NO_CHANGE}", urgency=txt.INF_CLR, tty=self.tty)



#!/usr/bin/env python
#
# This file is part of pacman-mirrors.
#
# pacman-mirrors is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# pacman-mirrors is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with pacman-mirrors.  If not, see <http://www.gnu.org/licenses/>.
#
# Authors: Frede Hundewadt <echo ZmhAbWFuamFyby5vcmcK | base64 -d>

"""Pacman-Mirrors Sort Mirros Functions"""
from operator import itemgetter


def sort_mirror_pool(
    worklist: list, field: str = "resp_time", reverse: bool = False
) -> list:
    """
    Sort the mirror pool
    :param worklist: list to be sorted
    :param field: field to sort on
    :param reverse: direction
    :return: new list sorted by field
    """
    result = sorted(worklist, key=itemgetter(field), reverse=reverse)
    # print(f"{result[0]}")
    # for x in result:
    #     print(f"resp_time => {x['resp_time']}")
    # exit()
    return result



#!/usr/bin/env python
#
# This file is part of pacman-mirrors.
#
# pacman-mirrors is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# pacman-mirrors is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with pacman-mirrors.  If not, see <http://www.gnu.org/licenses/>.
#
# Authors: Frede Hundewadt <echo ZmhAbWFuamFyby5vcmcK | base64 -d>

"""Pacman-Mirrors Filter Functions"""

from pacman_mirrors.constants import txt

# from constats.txt.py
# MIRROR RELATED
# LASTSYNC_OK = "24:00"  # last syncronize in the past 24 hours
# LASTSYNC_NA = "9800:00"  # last syncronization not available
# SERVER_BAD = "9999:99"  # default last syncronization status
# SERVER_RES = 99.99  # default response status


def filter_bad_mirrors(mirror_pool: list) -> list:
    """
    Remove known bad mirrors with last_sync == "9999:99" (status.json -1)
    :param mirror_pool:
    :return: list with bad mirrors removed
    """
    result = []
    mirrors = (x for x in mirror_pool if x["resp_time"] != txt.SERVER_BAD)
    for mirror in mirrors:
        result.append(mirror)

    return result


def filter_error_mirrors(mirror_pool: list) -> list:
    """
    Remove mirrors with resp_time == 99.99
    :param mirror_pool:
    :return: list with error mirrors removed
    """
    result = []
    mirrors = (x for x in mirror_pool if x["resp_time"] != txt.SERVER_RES)
    for mirror in mirrors:
        result.append(mirror)

    return result


def filter_poor_mirrors(mirror_pool: list, interval: int = 720) -> list:
    """
    Remove poorly updated mirrors last_sync is more than interval hours
    :param mirror_pool:
    :param interval: hours since last sync - defaults to 30 days
    :return: list with mirrors removed which has not synced since interval
    """
    result = []
    mirrors = (
        x for x in mirror_pool if int(str(x["last_sync"]).split(":")[0]) < interval
    )

    for mirror in mirrors:
        result.append(mirror)

    return result



#!/usr/bin/env python
#
# This file is part of pacman-mirrors.
#
# pacman-mirrors is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# pacman-mirrors is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with pacman-mirrors.  If not, see <http://www.gnu.org/licenses/>.
#
# Authors: Esclapion
#          Hugo Posnic <huluti@manjaro.org>

"""Pacman-Mirrors API Functions Module"""


import os
import sys
import tempfile

from pacman_mirrors.constants import txt
from pacman_mirrors.functions import util
from pacman_mirrors.functions import fileFn


def find_mirrorlist_branch(filename: str, tty: bool = False) -> str:
    """find and return the branch found in mirrorlist
    :param filename:
    :param tty:
    """
    try:
        with open(filename) as mirrorlist:
            for line in mirrorlist:
                if "Server = " in line:
                    workstring = line.strip()[-21:]  # /unstable/$repo/$arch
                    pos = workstring.find("/")
                    workstring = workstring[pos + 1 :]
                    pos = workstring.find("/")
                    return workstring[:pos]
    except OSError as err:
        util.msg(
            message=f"{txt.CANNOT_READ_FILE}: {err.filename}: {err.strerror}",
            urgency=txt.ERR_CLR,
            tty=tty,
        )

        sys.exit(2)


def normalize_config(filename: str) -> None:
    """Normalize configuration
    :param filename:
    """
    normalize_country(filename)
    normalize_method(filename)
    normalize_protocols(filename)
    normalize_ssl(filename)
    normalize_testfile(filename)


def normalize_country(filename: str) -> None:
    """Write default OnlyCountry =
    :param filename:
    """
    lookfor = "OnlyCountry ="
    default = "# OnlyCountry =\n"
    with open(filename) as cnf, tempfile.NamedTemporaryFile(
        "w+t", dir=os.path.dirname(filename), delete=False
    ) as tmp:
        replaced = False
        for line in cnf:
            if lookfor in line:
                tmp.write(f"{default}")
                replaced = True
            else:
                tmp.write(f"{line}")
        if not replaced:
            tmp.write(f"{default}")
    os.replace(tmp.name, filename)
    os.chmod(filename, 0o644)


def normalize_method(filename: str) -> None:
    """Write default Method = rank
    :param filename:
    """
    lookfor = "Method ="
    default = "# Method = rank\n"
    with open(filename) as cnf, tempfile.NamedTemporaryFile(
        "w+t", dir=os.path.dirname(filename), delete=False
    ) as tmp:
        replaced = False
        for line in cnf:
            if lookfor in line:
                tmp.write(f"{default}")
                replaced = True
            else:
                tmp.write(f"{line}")
        if not replaced:
            tmp.write(f"{default}")
    os.replace(tmp.name, filename)
    os.chmod(filename, 0o644)


def normalize_protocols(filename: str) -> None:
    """Write default Protocols =
    :param filename:
    """
    lookfor = "Protocols ="
    default = "# Protocols =\n"
    with open(filename) as cnf, tempfile.NamedTemporaryFile(
        "w+t", dir=os.path.dirname(filename), delete=False
    ) as tmp:
        replaced = False
        for line in cnf:
            if lookfor in line:
                tmp.write(f"{default}")
                replaced = True
            else:
                tmp.write(f"{line}")
        if not replaced:
            tmp.write(f"{default}")
    os.replace(tmp.name, filename)
    os.chmod(filename, 0o644)


def normalize_ssl(filename: str) -> None:
    """Write default SSLVerify = False
    :param filename:
    """
    default = "# SSLVerify = False\n"
    with open(filename) as cnf, tempfile.NamedTemporaryFile(
        "w+t", dir=os.path.dirname(filename), delete=False
    ) as tmp:

        replaced = False
        for line in cnf:
            if "SSLVerify =" in line:
                tmp.write(f"{default}")
                replaced = True
            else:
                tmp.write(f"{line}")
        if not replaced:
            tmp.write(f"{default}")

    os.replace(tmp.name, filename)
    os.chmod(filename, 0o644)


def normalize_testfile(filename: str) -> None:
    """Write default TestFile = core.db.tar.gz
    :param filename:
    """
    lookfor = "TestFile ="
    default = "# TestFile = core.db.tar.gz\n"
    with open(filename) as cnf, tempfile.NamedTemporaryFile(
        "w+t", dir=os.path.dirname(filename), delete=False
    ) as tmp:

        replaced = False
        for line in cnf:
            if lookfor in line:
                tmp.write(f"{default}")
                replaced = True
            else:
                tmp.write(f"{line}")
        if not replaced:
            tmp.write(f"{default}")

    os.replace(tmp.name, filename)
    os.chmod(filename, 0o644)


def sanitize_prefix(prefix: str) -> str:
    """Sanitize prefix
    :param prefix:
    :returns sanitized prefix
    """
    if prefix.endswith("/"):
        prefix = prefix[:-1]

    if not fileFn.check_file(prefix, True):
        exit(1)

    return prefix


def sanitize_url(url: str) -> str:
    """Sanitize url
    :param url:
    :returns sanitized url
    """
    if url.endswith("/"):
        return url
    return url + "/"


def write_config_branch(
    branch: str, filename: str, tty: bool = False, quiet: bool = False
) -> None:
    """Write branch
    :param branch:
    :param filename:
    :param tty:
    :param quiet:
    """
    if branch == "stable":
        branch = "# Branch = stable\n"
    else:
        branch = f"Branch = {branch}\n"

    try:
        with open(filename) as cnf, tempfile.NamedTemporaryFile(
            "w+t", dir=os.path.dirname(filename), delete=False
        ) as tmp:

            replaced = False
            for line in cnf:
                if "Branch =" in line:
                    tmp.write(branch)
                    replaced = True
                else:
                    tmp.write(f"{line}")

            if not replaced:
                tmp.write(branch)

        os.replace(tmp.name, filename)
        os.chmod(filename, 0o644)
        if not quiet:
            util.msg(message=f"{txt.API_CONF_RE_BRANCH}", urgency=txt.INF_CLR, tty=tty)

    except OSError as err:
        util.msg(
            message=f"{txt.CANNOT_READ_FILE}: {err.filename}: {err.strerror}",
            urgency=txt.ERR_CLR,
            tty=tty,
        )

        sys.exit(2)


def write_mirrorlist_branch(
    newbranch: str, filename: str, tty: bool = False, quiet: bool = False
) -> None:
    """Write branch in mirrorlist
    :param newbranch:
    :param filename:
    :param tty:
    :param quiet:
    """
    branch = find_mirrorlist_branch(filename=filename, tty=tty)
    try:
        with open(filename) as mirrorlist, tempfile.NamedTemporaryFile(
            "w+t", dir=os.path.dirname(filename), delete=False
        ) as tmp:
            for line in mirrorlist:
                if "Server =" in line:
                    line = line.replace(branch, newbranch)
                    tmp.write(f"{line}")
                else:
                    tmp.write(f"{line}")

        os.replace(tmp.name, filename)
        os.chmod(filename, 0o644)

        if not quiet:
            util.msg(
                message=f"{txt.API_MIRRORLIST_RE_BRANCH}", urgency=txt.INF_CLR, tty=tty
            )

    except OSError as err:
        util.msg(
            message=f"{txt.CANNOT_READ_FILE}: {err.filename}: {err.strerror}",
            urgency=txt.ERR_CLR,
            tty=tty,
        )

        sys.exit(2)


def write_protocols(
    protocols: str, filename: str, tty: bool = False, quiet: bool = False
) -> None:
    """Write protocols
    :param protocols:
    :param filename:
    :param tty:
    :param quiet:
    """
    if protocols:
        protocols = f"Protocols = {','.join(protocols)}\n"
    else:
        protocols = "# Protocols = \n"

    try:
        with open(filename) as cnf, tempfile.NamedTemporaryFile(
            "w+t", dir=os.path.dirname(filename), delete=False
        ) as tmp:

            replaced = False
            for line in cnf:
                if "Protocols =" in line:
                    tmp.write(protocols)
                    replaced = True
                else:
                    tmp.write(f"{line}")

            if not replaced:
                tmp.write(protocols)

        os.replace(tmp.name, filename)
        os.chmod(filename, 0o644)
        if not quiet:
            util.msg(message=f"{txt.API_CONF_PROTOCOLS}", urgency=txt.INF_CLR, tty=tty)

    except OSError as err:
        util.msg(
            message=f"{txt.CANNOT_READ_FILE}: {err.filename}: {err.strerror}",
            urgency=txt.ERR_CLR,
            tty=tty,
        )

        sys.exit(2)



#!/usr/bin/env python
#
# This file is part of pacman-mirrors.
#
# pacman-mirrors is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# pacman-mirrors is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with pacman-mirrors.  If not, see <http://www.gnu.org/licenses/>.
#
# from https://wiki.maemo.org/Internationalize_a_Python_application

"""Pacman-Mirrors Translation Module"""

import os
import sys
import locale
import gettext
import logging

#  The translation files will be under
#  @LOCALE_DIR@/@LANGUAGE@/LC_MESSAGES/@APP_NAME@.mo
APP_NAME = "pacman_mirrors"
APP_DIR = os.path.join(sys.prefix, "share")
LOCALE_DIR = os.path.join(APP_DIR, "locale")


def get_language():
    if locale.getlocale()[0] is not None:
        return locale.getlocale()[0]

    elif os.environ.get("LC_ALL") is not None:
        return os.environ.get("LC_ALL").split(".")[0]

    elif os.environ.get("LANGUAGE") is not None:
        return os.environ.get("LANGUAGE").split(".")[0]

    elif os.environ.get("LANG") is not None:
        return os.environ.get("LANG").split(".")[0]

    elif os.environ.get("GDM_LANG") is not None:
        return os.environ.get("GDM_LANG").split(".")[0]

    else:
        logging.warn(
            "No translations found for your language. Falling back to english."
        )
        return "en_US"


gettext.install(APP_NAME, LOCALE_DIR)
gettext.bindtextdomain(APP_NAME, LOCALE_DIR)
gettext.textdomain(APP_NAME)
language = gettext.translation(APP_NAME, LOCALE_DIR, [get_language()], fallback=True)

# Add this to every module:
#
# import i18n
# _ = i18n.language.gettext



#!/usr/bin/env python
#
# This file is part of pacman-mirrors.
#
# pacman-mirrors is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# pacman-mirrors is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with pacman-mirrors.  If not, see <http://www.gnu.org/licenses/>.
#
# Authors: Frede Hundewadt <echo ZmhAbWFuamFyby5vcmcK | base64 -d>

"""Manjaro-Mirrors File Functions"""

import datetime
import os
import sys
from pacman_mirrors.constants import colors as color
from pacman_mirrors.constants import txt
from pacman_mirrors.functions import jsonFn
from pacman_mirrors.functions import util


def check_file(filename: str, folder: bool = False) -> bool:
    """
    Check existence of named file
    :param filename:
    :param folder:
    :returns bool if existing
    """
    if folder:
        return os.path.isdir(filename)

    return os.path.isfile(filename)


def create_dir(foldername: str) -> None:
    """
    Create named folder if not exist
    :param foldername:
    """
    os.makedirs(foldername, mode=0o755, exist_ok=True)


def delete_file(filename: str) -> None:
    """
    Delete the named file if exist
    :param filename:
    :return:
    """
    if os.path.isfile(filename):
        os.remove(filename)


def return_mirror_filename(config: dict, tty: bool = False) -> tuple:
    """
    Find the mirror pool file
    :param config: config dictionary
    :param tty:
    :returns tuple with file and status
    """
    filename = ""
    # status.json or mirrors.json exist?
    status = False
    if check_file(config["status_file"]):
        status = True
        filename = config["status_file"]
    elif check_file(filename=config["mirror_file"]):
        filename = config["mirror_file"]

    if not filename:
        util.msg(message=f"\n{txt.HOUSTON}\n", tty=tty, color=color.RED)
        sys.exit(3)

    return filename, status


def write_mirror_list(
    config: dict,
    servers: list,
    tty: bool = False,
    custom: bool = False,
    quiet: bool = False,
    interactive: bool = False,
) -> None:
    """
    Write servers to /etc/pacman.d/mirrorlist
    :param config: configuration dictionary
    :param servers: list of servers to write
    :param tty:
    :param custom:
    :param quiet:
    :param interactive:
    """
    try:
        with open(config["mirror_list"], "w") as outfile:
            if not quiet:
                util.msg(
                    message=f"{txt.WRITING_MIRROR_LIST}", urgency=txt.INF_CLR, tty=tty
                )
            # write list header
            write_mirrorlist_header(outfile, custom=custom)
            cols, lines = util.terminal_size()
            for server in servers:
                url = server["url"]
                protocol = server["protocols"][0]
                pos = url.find(":")
                server["url"] = (
                    f'{protocol}{url[pos:]}{config["branch"]}{config["repo_arch"]}'
                )

                if server["resp_time"] == 99.99:
                    # do not write bad servers to mirrorlist
                    continue

                if interactive:
                    if not quiet:
                        message = f'{server["country"]:<15} : {server["url"]}'
                        util.msg(message=f"{message:.{cols}}", tty=tty)
                else:
                    msg_url = f'{protocol}{url[pos:]}{config["branch"]}'
                    if not quiet:
                        message = f'{server["country"]:<15} : {msg_url}'
                        util.msg(message=f"{message:.{cols}}", tty=tty)

                # write list entry
                write_mirrorlist_entry(outfile, server)

            if not quiet:
                util.msg(
                    message=f'{txt.MIRROR_LIST_SAVED}: {config["mirror_list"]}',
                    urgency=txt.INF_CLR,
                    tty=tty,
                )

    except OSError as err:
        util.msg(
            message=f"{txt.CANNOT_WRITE_FILE}: {err.filename}: {err.strerror}",
            urgency=txt.ERR_CLR,
            tty=tty,
        )

        sys.exit(2)


def read_mirror_file(filename: str) -> list:
    """
    Read content of named file - json data assumed
    :param filename:
    :returns: list of mirrors
    """
    return jsonFn.read_json_file(filename, use_ordered_dict=True)


def write_mirrorlist_header(handle, custom: bool = False) -> None:
    """
    Write mirrorlist header
    :param handle: handle to a file opened for writing
    :param custom: controls content of the header
    """
    # handle creation time in unicode
    # http://stackoverflow.com/questions/16034060/
    #  python3-datetime-datetime-strftime-failed-to-accept-utf-8-string-format
    generated_timestamp = datetime.datetime.now().strftime("%Y-%m-%d %H:%M")
    handle.write("##\n")
    if custom:
        handle.write(
            f"## Manjaro Linux {txt.MIRROR_LIST_CUSTOM_HEADER}\n"
            f"## {txt.MIRROR_LIST_GENERATED_ON} {generated_timestamp}\n"
            "##\n"
            f"## {txt.PLEASE_USE} '{txt.MODIFY_CUSTOM}' {txt.MIRROR_LIST_CUSTOM_RESET}\n"
            f"## {txt.PLEASE_USE} '{txt.RESET_ALL}' {txt.MIRROR_LIST_CUSTOM_RESET}\n"
            f"## {txt.REMOVE_CUSTOM_CONFIG} '{txt.RESET_ALL}'\n"
        )
    else:
        handle.write(
            f"## Manjaro Linux {txt.MIRROR_LIST_DEFAULT_HEADER}\n"
            f"## {txt.MIRROR_LIST_GENERATED_ON} {generated_timestamp}\n"
            "##\n"
            f"## {txt.PLEASE_USE} '{txt.MODIFY_DEFAULT} [{txt.NUMBER}]' {txt.MIRROR_LIST_DEFAULT_MODIFY}\n"
            f"## ({txt.USE_ZERO_FOR_ALL})\n"
        )

    handle.write("##\n\n")


def write_mirrorlist_entry(handle, mirror: dict) -> None:
    """
    Write mirror to mirror list or file
    :param handle: handle to a file opened for writing
    :param mirror: mirror object
    """
    workitem = mirror
    handle.write(f'## Country : {workitem["country"]}\n')
    handle.write(f'Server = {workitem["url"]}\n\n')



#!/usr/bin/env python
#
# This file is part of pacman-mirrors.
#
# pacman-mirrors is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# pacman-mirrors is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with pacman-mirrors.  If not, see <http://www.gnu.org/licenses/>.
#
# Authors: Esclapion
#          Hugo Posnic <huluti@manjaro.org>

"""Pacman-Mirrors GTK GUI Module"""
import gi

gi.require_version("Gtk", "3.0")
from gi.repository import Gtk
from operator import itemgetter
from random import shuffle
from pacman_mirrors.constants import txt
from pacman_mirrors.translation import i18n

_ = i18n.language.gettext


class GraphicalUI(Gtk.Window):
    def __init__(self, server_list: list, random: bool, default: bool):
        title = txt.I_TITLE_RANDOM if random else txt.I_TITLE
        if default:
            title = "Manjaro Mirrors"

        try:
            self.gtk_init = True
            Gtk.Window.__init__(self, title=title)
            self.random = random
            self.set_size_request(700, 350)
            self.set_border_width(10)
            self.set_position(Gtk.WindowPosition.CENTER)

            custom_mirrors = []
            for server in server_list:
                try:
                    _ = server_list[0]
                    custom_mirrors.append(
                        (False, server["country"], server["last_sync"], server["url"])
                    )

                except IndexError as i:
                    print(f"{txt.ERR_CLR} IndexError -> {i}")
                except KeyError as k:
                    print(f"{txt.ERR_CLR} KeyError -> {k}")

            self.store = Gtk.ListStore(bool, str, str, str)
            for mirror_ref in custom_mirrors:
                self.store.append(list(mirror_ref))

            scrolled_tree = Gtk.ScrolledWindow()
            self.tree = Gtk.TreeView(self.store, vexpand=True)
            renderer = Gtk.CellRendererToggle()
            renderer.connect("toggled", self.on_toggle)
            column = Gtk.TreeViewColumn(txt.I_USE, renderer, active=0)
            self.tree.append_column(column)
            renderer = Gtk.CellRendererText()
            column = Gtk.TreeViewColumn(txt.I_COUNTRY, renderer, text=1)
            column.set_sort_column_id(1)
            self.tree.append_column(column)
            renderer = Gtk.CellRendererText()
            column = Gtk.TreeViewColumn(txt.I_LAST_SYNC, renderer, text=2)
            column.set_sort_column_id(2)
            self.tree.append_column(column)
            renderer = Gtk.CellRendererText()
            column = Gtk.TreeViewColumn(txt.I_URL, renderer, text=3)
            column.set_sort_column_id(3)
            self.tree.append_column(column)
            scrolled_tree.add(self.tree)
            header = Gtk.Label(txt.I_LIST_TITLE)
            button_cancel = Gtk.Button(txt.I_CANCEL)
            button_cancel.connect("clicked", self.cancel)
            self.button_done = Gtk.Button(txt.I_CONFIRM, sensitive=False)
            self.button_done.connect("clicked", self.done)
            grid = Gtk.Grid(column_homogeneous=True, column_spacing=10, row_spacing=10)
            grid.attach(header, 0, 0, 2, 1)
            grid.attach(scrolled_tree, 0, 1, 2, 1)
            grid.attach(button_cancel, 0, 2, 1, 1)
            grid.attach(self.button_done, 1, 2, 1, 1)
            self.add(grid)
            self.server_list = server_list
            self.custom_list = []
            self.is_done = False

        except RuntimeError:
            self.gtk_init = False

    def on_toggle(self, widget, path):
        """Add or remove server from custom list"""
        self.store[path][0] = not self.store[path][0]
        if self.store[path][0]:
            for server in self.server_list:
                if server["url"] == self.store[path][3]:
                    self.custom_list.append(server)
        else:
            for server in self.custom_list:
                if server["url"] == self.store[path][3]:
                    self.custom_list.remove(server)

        self.button_done.set_sensitive(bool(self.custom_list))

    def cancel(self, button):
        """Cancel mirrorlist"""
        self.custom_list = []
        self.is_done = True
        Gtk.main_quit()

    def done(self, button):
        """Confirm choice"""
        dialog = Gtk.Dialog(
            txt.I_CONFIRM_SELECTION,
            None,
            0,
            (
                Gtk.STOCK_CANCEL,
                Gtk.ResponseType.CANCEL,
                Gtk.STOCK_OK,
                Gtk.ResponseType.OK,
            ),
        )

        dialog.set_transient_for(self)
        dialog.set_border_width(10)
        box = dialog.get_content_area()
        box.set_spacing(10)
        box.add(Gtk.Label(txt.I_USE_THESE_MIRRORS))
        dialog.show_all()
        response = dialog.run()

        if response == Gtk.ResponseType.OK:
            # Quit GUI
            dialog.destroy()
            for line in self.custom_list:
                line["last_sync"] = (
                    line["last_sync"]
                    .replace(" ", ":")
                    .replace("h", "")
                    .replace("m", "")
                )

            if self.random:
                shuffle(self.custom_list)
            else:
                self.custom_list.sort(key=itemgetter("resp_time"))

            self.is_done = True
            Gtk.main_quit()

        elif response == Gtk.ResponseType.CANCEL:
            dialog.destroy()  # Go back to selection


def run(server_list: list, random: bool, default: bool = False) -> object:
    window = GraphicalUI(server_list, random, default)
    if window.gtk_init:
        window.connect("delete-event", Gtk.main_quit)
        window.show_all()
        Gtk.main()
        return window

    return window



#!/usr/bin/env python
#
# This file is part of pacman-mirrors.
#
# pacman-mirrors is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# pacman-mirrors is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with pacman-mirrors.  If not, see <http://www.gnu.org/licenses/>.
#
# Authors: Frede Hundewadt <echo ZmhAbWFuamFyby5vcmcK | base64 -d>

"""Manjaro-Mirrors HTTP Functions"""

import filecmp
import json
import os
import ssl
import time
import shutil
from urllib import request
from urllib.error import URLError
from contextlib import closing
import subprocess
import requests
from pacman_mirrors import __version__
from pacman_mirrors.config import configuration as conf
from pacman_mirrors.constants import txt
from pacman_mirrors.functions import fileFn
from pacman_mirrors.functions import jsonFn
from pacman_mirrors.functions import util


def headers():
    return {"User-Agent": f"{conf.USER_AGENT}/{__version__}"}


def check_timestamp(url: str) -> bool:
    """Use a head request to get the last modified time of the mirror pool"""
    resp = requests.head(url, headers=headers())
    resp.raise_for_status()
    remote_file_stamp = resp.headers.get("Last-Modified")
    local_file_stamp = os.path.getmtime(conf.MIRROR_FILE)
    return (
        remote_file_stamp is not None
        and local_file_stamp is not None
        and remote_file_stamp == local_file_stamp
    )


def download_mirrors(config: dict) -> tuple:
    """Retrieve mirrors from manjaro.org
    :param config:
    :returns: tuple with bool for mirrors.json and status.json
    :rtype: tuple
    """
    fetchmirrors = True
    fetchstatus = True
    message = ""
    try:
        util.msg(
            message=f"=> Mirror pool: {config['url_mirrors_json']}", urgency=txt.INF_CLR
        )
        # mirrors.json
        if not check_timestamp(config["url_mirrors_json"]):
            resp = requests.get(
                url=config["url_mirrors_json"],
                headers=headers(),
                timeout=config["timeout"],
            )

            resp.raise_for_status()
            mirrorlist = resp.json()
            tempfile = config["work_dir"] + "/.temp.file"
            jsonFn.json_dump_file(mirrorlist, tempfile)
            filecmp.clear_cache()

            if fileFn.check_file(conf.WORK_DIR, folder=True):
                if not fileFn.check_file(config["mirror_file"]):
                    jsonFn.json_dump_file(mirrorlist, config["mirror_file"])
                elif not filecmp.cmp(tempfile, config["mirror_file"]):
                    jsonFn.json_dump_file(mirrorlist, config["mirror_file"])

            os.remove(tempfile)

    except (json.JSONDecodeError,) as err:
        message = f"Invalid JSON data: {err}"
    except (requests.exceptions.ConnectionError,) as err:
        message = f"Connection: {err}"
    except (requests.exceptions.Timeout,) as err:
        message = f"Connection: {err}"
    except (requests.exceptions.HTTPError,) as err:
        message = f"Connection {err}"

    if message != "":
        fetchmirrors = False
        util.msg(message=message, urgency=txt.ERR_CLR, newline=True)
        message = ""

    # fetch status.json
    try:
        util.msg(
            message=f"=> Mirror status: {config['url_status_json']}",
            urgency=txt.INF_CLR,
        )
        resp = requests.get(
            url=config["url_status_json"],
            headers=headers(),
            timeout=config["timeout"],
        )

        statuslist = resp.json()
        jsonFn.write_json_file(statuslist, config["status_file"])

    except (json.JSONDecodeError,) as err:
        message = f"Invalid JSON data: {err}"
    except (requests.exceptions.ConnectionError,) as err:
        message = f"Connection: {err}"
    except (requests.exceptions.Timeout,) as err:
        message = f"Connection: {err}"
    except (requests.exceptions.HTTPError,) as err:
        message = f"Connection {err}"

    if message != "":
        fetchstatus = False
        util.msg(message=message, urgency=txt.ERR_CLR, newline=True)

    return fetchmirrors, fetchstatus


def get_ip_country(timeout: int = 2) -> str:
    """
    Get the user country from connection IP (might be VPN who knows)
    :return: country name
    """
    try:
        url = "https://get.geojs.io/v1/ip/country/full"
        resp = requests.get(url, timeout=timeout)
        resp.raise_for_status()

    except requests.exceptions.ConnectionError:
        return ""

    return resp.text


def get_http_response(url: str, timeout: int) -> str:
    """
    Used for http mirrors
    :return: bool
    """
    message = ""
    try:
        resp = requests.get(url=url, headers=headers(), timeout=timeout)
        resp.raise_for_status()
        # _ = resp.text
    except (requests.exceptions.ConnectionError,) as err:
        message = f"Connection: {err}"
    except (requests.exceptions.Timeout,) as err:
        message = f"Connection: {err}"
    except (requests.exceptions.HTTPError,) as err:
        message = f"Connection {err}"
    return message


def get_ftp_response(url: str, timeout: int) -> str:
    """
    Used for ftp response
    :return:
    """
    message = ""
    try:
        with closing(request.urlopen(url, timeout=timeout)) as ftpReq:
            with open(conf.WORK_DIR + "/.testresponse", "wb") as testFile:
                shutil.copyfileobj(ftpReq, testFile)

            os.remove(conf.WORK_DIR + "/.testresponse")

    except URLError as e:
        if e.reason.find("No such file or directory") >= 0:
            message = "FileNotFound"
        else:
            message = f"{e.reason}"

    return message


def get_mirror_response(
    url: str,
    config: dict,
    tty: bool = False,
    timeout: int = 2,
    count: int = 1,
    quiet: bool = False,
    ssl_verify: bool = True,
) -> float:
    """Query mirror by downloading a file and measuring the time taken
    :param config:
    :param ssl_verify:
    :param tty:
    :param url:
    :param timeout:
    :param count:
    :param quiet:
    :returns: always return a float value with response time
    """
    response_time = txt.SERVER_RES  # prepare default return value
    probe_stop = None
    message = ""
    context = ssl.create_default_context()
    arch = "x86_64"

    if config["arm"]:
        arch = "aarch64"

    probe_url = f"{url}{config['branch']}/core/{arch}/{config['test_file']}"

    if not ssl_verify:
        context.check_hostname = False
        context.verify_mode = ssl.CERT_NONE

    probe_start = time.time()
    if probe_url.startswith("http"):
        message = get_http_response(url=probe_url, timeout=timeout)
        probe_stop = time.time()

    if probe_url.startswith("ftp"):
        message = get_ftp_response(url=probe_url, timeout=timeout)
        probe_stop = time.time()

    if message and not quiet:
        util.msg(message=f"{message}", urgency=txt.ERR_CLR, tty=tty, newline=True)

    if probe_stop and not message:
        response_time = round((probe_stop - probe_start), 3)

    return response_time


def check_internet_connection(tty: bool = False, timeout: int = 2) -> bool:
    """Check for internet connection
    @param timeout:
    @param tty:
    """

    def output_err(hostname, error, emit: bool = True):
        if emit:
            print("")
            util.msg(f"{hostname} '{error}'", urgency=txt.WRN_CLR, tty=tty)
            print("")

    for host in conf.INET_CONN_CHECK_URLS:
        try:
            requests.get(host, headers=headers(), timeout=timeout)
            return True
        except (requests.exceptions.HTTPError, requests.exceptions.SSLError) as err:
            # this indicates an active internet connection
            # otherwilse http errors would not be possible
            # this can return true
            # an internet connection is available
            # the host didn't respond well to https request
            return True
        except (
            requests.exceptions.ConnectionError,
            requests.exceptions.Timeout,
        ) as err:
            # this indicates that internet connection is not available
            # until the configuration of
            # the ping server is corrected
            # we could neglect the error
            return False
        except Exception as err:
            output_err(host, err, emit=True)
            return False

    return False


def ping_host(host: str, tty: bool = False, count: int = 1) -> bool:
    """Check a host's availability
    :param host:
    :param count:
    :param tty:
    :rtype: boolean
    """
    util.msg(f"ping {host} x {count}", urgency=txt.INF_CLR, tty=tty)
    return subprocess.Popen(f"ping -c{count} {host} > /dev/null", shell=True) == 0


def download_mirror_pool(config: dict, tty: bool = False, quiet: bool = False) -> tuple:
    """Download updates from repo.manjaro.org
    :param config:
    :param quiet:
    :param tty:
    :returns: tuple with True/False for mirrors.json and status.json
    :rtype: tuple
    """
    result = None
    connected = check_internet_connection(tty=tty)
    if connected:
        if not quiet:
            util.msg(
                message=f"{txt.DOWNLOADING_MIRROR_FILE} Manjaro",
                urgency=txt.INF_CLR,
                tty=tty,
            )

        result = download_mirrors(config)
    else:
        if not fileFn.check_file(config["status_file"]):
            if not quiet:
                util.msg(
                    message=f"{txt.MIRROR_FILE} {config['status_file']} {txt.IS_MISSING}",
                    urgency=txt.WRN_CLR,
                    tty=tty,
                )

                util.msg(
                    message=f"{txt.FALLING_BACK} {conf.MIRROR_FILE}",
                    urgency=txt.WRN_CLR,
                    tty=tty,
                )

            result = (True, False)

        if not fileFn.check_file(config["mirror_file"]):
            if not quiet:
                util.msg(message=f"{txt.HOUSTON}", urgency=txt.HOUSTON, tty=tty)

            result = (False, False)

    return result



#!/usr/bin/env python
#
# This file is part of pacman-mirrors.
#
# pacman-mirrors is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# pacman-mirrors is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with pacman-mirrors.  If not, see <http://www.gnu.org/licenses/>.
#
# Authors: Frede Hundewadt <echo ZmhAbWFuamFyby5vcmcK | base64 -d>

# http constants
TEST_FILE = "core.db.tar.gz"
USER_AGENT = "PacmanMirrors"
URL_MIRROR_JSON = "https://repo.manjaro.org/mirrors.json"
URL_STATUS_JSON = "https://repo.manjaro.org/status.json"
INET_CONN_CHECK_URLS = [
    "http://ping.manjaro.org/pacmanmirrors",
    "https://wikipedia.org",
    "https://archlinux.org",
]
# there is no certificate on the ping url
# we have been using https for everything
# need to set up a certificate for the ping domain
# STATISTIC_HOSTNAME = "http://ping.manjaro.org/pacmanmirrors"
CONFIG_FILE = "/etc/pacman-mirrors.conf"
MIRROR_LIST = "/etc/pacman.d/mirrorlist"
# /var/lib
# WORK_DIR = "/var/lib/pacman-mirrors"
WORK_DIR = "/var/lib/pacman-mirrors"
CUSTOM_FILE = f"{WORK_DIR}/custom-mirrors.json"
MIRROR_FILE = f"{WORK_DIR}/mirrors.json"
STATUS_FILE = f"{WORK_DIR}/status.json"
# repo constants
BRANCHES = ("stable", "testing", "unstable")
ARM_BRANCHES = ("arm-stable", "arm-testing", "arm-unstable")
PROTOCOLS = ("https", "http", "ftp")
METHODS = ("rank", "random")
SSL = ("True", "False")
REPO_ARCH = "/$repo/$arch"



#!/usr/bin/env python
#
# This file is part of pacman-mirrors.
#
# pacman-mirrors is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# pacman-mirrors is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with pacman-mirrors.  If not, see <http://www.gnu.org/licenses/>.
#
# Authors: Frede Hundewadt <echo ZmhAbWFuamFyby5vcmcK | base64 -d>

"""Pacman-Mirrors Common Mirror List Builder Module"""

from random import shuffle
from pacman_mirrors.builder.builder import build_pool
from pacman_mirrors.constants import txt
from pacman_mirrors.functions import util
from pacman_mirrors.functions.sortMirrorFn import sort_mirror_pool
from pacman_mirrors.functions.testMirrorFn import test_mirror_pool
from pacman_mirrors.functions.outputFn import (
    write_pacman_mirror_list,
    write_custom_mirrors_json,
)


def build_mirror_list(self) -> None:
    """
    Builds a list of mirror servers based on the configuration and selected countries.

    This function processes a pool of available mirrors, selects or ranks them based
    on user preferences and configurations, and writes the finalized list to the appropriate
    configuration file. When using custom methods, appropriate messages are displayed to inform
    the user about required manual actions.

    Raises:
        IndexError: Raised if an empty list is accessed during processing steps.

    Parameters:
        None

    Returns:
        None
    """
    work_pool = build_pool(self)

    # Check selected_countries against the full country list
    # If selected_countries is the lesser of the two, then we build a custom pool file
    if len(self.selected_countries) < len(self.mirrors.country_pool):
        if self.custom:
            try:
                _ = self.selected_countries[0]
                write_custom_mirrors_json(self=self, selected_mirrors=work_pool)
            except IndexError:
                print("mirror pool is empty")

    if self.config["method"] == "rank":
        # TODO variables mirror function local and each other
        work_pool = test_mirror_pool(self=self, worklist=work_pool)
        work_pool = sort_mirror_pool(
            worklist=work_pool, field="resp_time", reverse=False
        )
    else:
        shuffle(work_pool)

    # Write the mirror list
    try:
        _ = work_pool[0]
        write_pacman_mirror_list(self=self, selected_servers=work_pool)
        if self.custom:
            util.msg(
                message=f"{txt.MIRROR_LIST_CUSTOM_RESET} 'sudo {txt.MODIFY_CUSTOM}'",
                urgency=txt.INF_CLR,
                tty=self.tty,
            )
            util.msg(
                message=f"{txt.REMOVE_CUSTOM_CONFIG} 'sudo {txt.RESET_ALL}'",
                urgency=txt.INF_CLR,
                tty=self.tty,
            )

        if self.no_status:
            util.msg(
                message=f"{txt.OVERRIDE_STATUS_CHOICE}",
                urgency=txt.WRN_CLR,
                tty=self.tty,
            )
            util.msg(
                message=f"{txt.OVERRIDE_STATUS_MIRROR}",
                urgency=txt.WRN_CLR,
                tty=self.tty,
            )

    except IndexError:
        util.msg(message=f"{txt.NO_SELECTION}", urgency=txt.WRN_CLR, tty=self.tty)
        util.msg(message=f"{txt.NO_CHANGE}", urgency=txt.INF_CLR, tty=self.tty)



#!/usr/bin/env python
#
# This file is part of pacman-mirrors.
#
# pacman-mirrors is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# pacman-mirrors is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with pacman-mirrors.  If not, see <http://www.gnu.org/licenses/>.
#
# Authors: Frede Hundewadt <echo ZmhAbWFuamFyby5vcmcK | base64 -d>

from pacman_mirrors.functions.filter_mirror_pool_functions import (
    filter_mirror_protocols,
    filter_mirror_country,
    filter_user_branch,
)

from pacman_mirrors.functions.filter_mirror_status_functions import (
    filter_bad_mirrors,
    filter_error_mirrors,
    filter_poor_mirrors,
)


def build_pool(self) -> list:
    """
    Builds and returns a filtered list of mirrors based on several criteria.

    The method processes a list of mirrors by applying various filters such as
    removing bad and error-prone mirrors, filtering based on country, protocol,
    and user configuration. It optionally considers factors like synchronization
    interval and user branches in the filtering process.

    Returns:
        list: A list of mirrors that match the applied filters.

    Raises:
        IndexError: Raised when protocol configuration is accessed without any
        available protocols.
    """

    # remove known bad mirrors - last sync 9999:99
    work_pool = filter_bad_mirrors(mirror_pool=self.mirrors.mirror_pool)

    # remove known error mirrors - response time 99.99
    work_pool = filter_error_mirrors(mirror_pool=work_pool)

    # Apply country filter if not fasttrack
    if not self.fasttrack:
        work_pool = filter_mirror_country(
            mirror_pool=work_pool, country_pool=self.selected_countries
        )

    # Apply protocol filter
    try:
        _ = self.config["protocols"][0]
        work_pool = filter_mirror_protocols(
            mirror_pool=work_pool, protocols=self.config["protocols"]
        )

    except IndexError:
        pass

    # Apply --no-status argument if applicable
    if self.no_status:
        # Apply interval filter
        if self.interval:
            work_pool = filter_poor_mirrors(
                mirror_pool=work_pool, interval=self.interval
            )
    else:
        work_pool = filter_user_branch(mirror_pool=work_pool, config=self.config)

    return work_pool



#!/usr/bin/env python
#
# This file is part of pacman-mirrors.
#
# pacman-mirrors is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# pacman-mirrors is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with pacman-mirrors.  If not, see <http://www.gnu.org/licenses/>.
#
# Authors: Frede Hundewadt <echo ZmhAbWFuamFyby5vcmcK | base64 -d>

"""Pacman-Mirrors API Handler Module"""

import shutil
import sys

from pacman_mirrors.constants import txt
from pacman_mirrors.api import apifn
from pacman_mirrors.functions import fileFn
from pacman_mirrors.functions import util


def set_config(
    self,
    set_pfx: str = None,
    set_branch: str = None,
    re_branch: bool = False,
    set_protocols: bool = False,
    set_url: str = None,
) -> None:
    """
    Api configuration function
    :param self:
    :param set_pfx: prefix to the config paths
    :param set_branch: replace branch in pacman-mirrors.conf
    :param re_branch: replace branch in mirror list
    :param set_protocols: replace protocols in pacman-mirrors.conf
    :param set_url: replace mirror url in mirror list
    """
    if set_url is None:
        set_url = ""

    if set_pfx is None:
        set_pfx = ""

    # apply api configuration to internal configuration object
    # Apply prefix if present
    if set_pfx:
        set_pfx = apifn.sanitize_prefix(set_pfx)
        self.config["config_file"] = set_pfx + self.config["config_file"]
        self.config["custom_file"] = set_pfx + self.config["custom_file"]
        self.config["mirror_file"] = set_pfx + self.config["mirror_file"]
        self.config["mirror_list"] = set_pfx + self.config["mirror_list"]
        self.config["status_file"] = set_pfx + self.config["status_file"]
        self.config["work_dir"] = set_pfx + self.config["work_dir"]

    # First API task: Set branch
    if set_branch:
        # Apply branch to internal config
        self.config["branch"] = set_branch
        util.aarch64_check(self=self, write=False)
        # pacman-mirrors.conf could absent so check for it
        if not fileFn.check_file(filename=self.config["config_file"]):
            # Copy from host system
            fileFn.create_dir(set_pfx + "/etc")
            shutil.copyfile("/etc/pacman-mirrors.conf", self.config["config_file"])
            # Normalize config
            apifn.normalize_config(filename=self.config["config_file"])
        # Write branch to config
        apifn.write_config_branch(
            branch=self.config["branch"],
            filename=self.config["config_file"],
            tty=self.tty,
            quiet=self.quiet,
        )
    # Second API task: Create a mirror list
    if set_url:
        # mirror list dir could absent so check for it
        fileFn.create_dir(foldername=set_pfx + "/etc/pacman.d")
        mirror = [
            {
                "url": apifn.sanitize_url(set_url),
                "country": "BUILDMIRROR",
                "protocols": [set_url[: set_url.find(":")]],
                "resp_time": "00.00",
            }
        ]
        fileFn.write_mirror_list(
            config=self.config, servers=mirror, tty=self.tty, quiet=self.quiet
        )
        # exit gracefully
        sys.exit(0)
    # Third API task: Write protocols to config
    if set_protocols:
        apifn.write_protocols(
            protocols=self.config["protocols"],
            filename=self.config["config_file"],
            tty=self.tty,
            quiet=self.quiet,
        )
    # Fourth API task: Rebranch mirror list
    if re_branch:
        if not set_branch:
            util.msg(
                message=f"{txt.API_ERROR_BRANCH}", urgency=txt.ERR_CLR, tty=self.tty
            )
            # print(".: {} {}".format(txt.ERR_CLR, txt.API_ERROR_BRANCH))
            sys.exit(1)
        apifn.write_mirrorlist_branch(
            newbranch=self.config["branch"],
            filename=self.config["config_file"],
            tty=self.tty,
            quiet=self.quiet,
        )



#!/usr/bin/env python
#
# This file is part of pacman-mirrors.
#
# pacman-mirrors is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# pacman-mirrors is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with pacman-mirrors.  If not, see <http://www.gnu.org/licenses/>.
#
# Authors: Frede Hundewadt <echo ZmhAbWFuamFyby5vcmcK | base64 -d>

"""Pacman-Mirrors Configuration Functions"""

import sys
from typing import Dict, Any, Tuple, List

from pacman_mirrors.config import configuration as conf
from pacman_mirrors.constants import txt
from pacman_mirrors.functions import util


def setup_config(self) -> tuple:
    """Get config information
    :returns: config, custom
    :rtype: tuple
    """
    custom = False
    # default config
    config: dict[str | Any, str | tuple[str, str, str] | list[Any] | bool | Any] = {
        "arm": False,
        "branch": "stable",
        "branches": conf.BRANCHES,
        "config_file": conf.CONFIG_FILE,
        "country_pool": [],
        "custom_file": conf.CUSTOM_FILE,
        "enterprise": False,
        "method": "rank",
        "mirror_file": conf.MIRROR_FILE,
        "mirror_list": conf.MIRROR_LIST,
        "no_update": False,
        "protocols": [],
        "repo_arch": conf.REPO_ARCH,
        "ssl_verify": True,
        "static": "",
        "statistic": False,
        "status_file": conf.STATUS_FILE,
        "test_file": conf.TEST_FILE,
        "timeout": 2,
        "url_mirrors_json": conf.URL_MIRROR_JSON,
        "url_status_json": conf.URL_STATUS_JSON,
        "work_dir": conf.WORK_DIR,
    }
    # try to replace default entries by reading conf file
    try:
        with open(config["config_file"]) as conf_file:
            for line in conf_file:
                line = line.strip()
                if line.startswith("#") or "=" not in line:
                    continue

                (key, value) = line.split("=", 1)
                key = key.rstrip()
                value = value.lstrip()
                if key and value:
                    if value.startswith('"') and value.endswith('"'):
                        value = value[1:-1]

                    if key == "Method":
                        config["method"] = value

                    if key == "Branch":
                        config["branch"] = value

                    if key == "Protocols":
                        if "," in value:
                            config["protocols"] = value.split(",")
                        else:
                            config["protocols"] = value.split(" ")

                    if key == "SSLVerify":
                        config["ssl_verify"] = value.lower().capitalize()

                    if key == "TestFile":
                        config["test_file"] = value
                        if not config["test_file"]:
                            config["test_file"] = conf.TEST_FILE

                    if key == "Static":
                        config["enterprise"] = True
                        config["static"] = value

                    if key == "Statistic":
                        config["statistic"] = True

    except (PermissionError, OSError) as err:
        util.msg(
            message=f"{txt.CANNOT_READ_FILE}: {err.filename}: {err.strerror}",
            urgency=txt.ERR_CLR,
            tty=self.tty,
        )

        sys.exit(2)

    return config, custom


def sanitize_config(config: dict) -> bool:
    """
    Verify configuration
    :param config:
    """
    errors = []
    if config["method"] not in conf.METHODS:
        errors.append(
            f"     'Method = {config['method']}'; {txt.EXP_CLR} {'|'.join(conf.METHODS)}"
        )

    if config["arm"]:
        if config["branch"] not in conf.ARM_BRANCHES:
            errors.append(
                "     'Branch = {}'; {} {}".format(
                    config["branch"], txt.EXP_CLR, "|".join(conf.ARM_BRANCHES)
                )
            )
    else:
        if config["branch"] not in conf.BRANCHES:
            errors.append(
                "     'Branch = {}'; {} {}".format(
                    config["branch"], txt.EXP_CLR, "|".join(conf.BRANCHES)
                )
            )

    if str(config["ssl_verify"]) not in conf.SSL:
        errors.append(
            "     'SSLVerify = {}'; {} {}".format(
                config["ssl_verify"], txt.EXP_CLR, "|".join(conf.SSL)
            )
        )

    for p in config["protocols"]:
        if p not in conf.PROTOCOLS:
            errors.append(
                "     'Protocols = {}'; {} {}".format(
                    config["protocols"], txt.EXP_CLR, ",".join(conf.PROTOCOLS)
                )
            )

    if config["enterprise"]:
        # check static mirror - simple validation
        if "://" or "." not in config["static"]:
            errors.append(
                "     'Static = {}'; {} (e.g. 'https://mirror1.tld/manjaro/')".format(
                    config["static"], txt.EXP_CLR
                )
            )

    if len(errors) != 0:
        print(f".: {txt.ERR_CLR}: {txt.INVALID_SETTING_IN} `{conf.CONFIG_FILE}`")
        for e in errors:
            print(e)
        return False
    return True



#!/usr/bin/env python
#
# This file is part of pacman-mirrors.
#
# pacman-mirrors is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# pacman-mirrors is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with pacman-mirrors.  If not, see <http://www.gnu.org/licenses/>.
#
# Authors: Frede Hundewadt <echo ZmhAbWFuamFyby5vcmcK | base64 -d>

"""Pacman-Mirrors Filter Functions"""

from pacman_mirrors.config import configuration as conf


def filter_mirror_country(mirror_pool: list, country_pool: list) -> list:
    """
    Return new mirror pool with selected countries
    :param mirror_pool:
    :param country_pool:
    :rtype: list
    """
    result = []
    for mirror in mirror_pool:
        if mirror["country"] in country_pool:
            result.append(mirror)

    return result


def filter_mirror_protocols(mirror_pool: list, protocols: list) -> list:
    """
    Return a new mirrorlist with protocols
    :type mirror_pool: list
    :type protocols: list
    :rtype: list
    """
    result = []
    for mirror in mirror_pool:
        accepted = []
        for idx, protocol in enumerate(protocols):
            if protocol in mirror["protocols"]:
                accepted.append(protocol)

        if accepted:
            mirror["protocols"] = accepted
            result.append(mirror)

    return result


def filter_user_branch(mirror_pool: list, config: dict) -> list:
    """
    Filter mirrorlist on users branch and branch sync state
    :param mirror_pool:
    :param config:
    :return: list of up-to-date mirrors
    """
    for idx, branch in enumerate(conf.BRANCHES):
        if config["arm"]:
            config_branch = config["branch"][4:]
        else:
            config_branch = config["branch"]

        if branch == config_branch:
            filtered = []
            for mirror in mirror_pool:
                try:
                    if mirror["branches"][idx] == 1:
                        filtered.append(mirror)
                except IndexError:
                    pass

            if len(filtered) > 0:
                return filtered

    return mirror_pool



#!/usr/bin/env python
#
# This file is part of pacman-mirrors.
#
# pacman-mirrors is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published b
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# pacman-mirrors is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with pacman-mirrors.  If not, see <http://www.gnu.org/licenses/>.
#
# Authors: Frede Hundewadt <echo ZmhAbWFuamFyby5vcmcK | base64 -d>

"""Pacman-Mirrors Country Functions"""
import os
from pacman_mirrors.functions.validFn import country_list_is_valid
from pacman_mirrors.functions.util import get_zone_countries

countries = get_zone_countries()


def wash_country_list(country_list: list):
    """
    Cleans and standardizes a list of country identifiers.

    This function processes a list of country identifiers, such as country names
    or codes, and standardizes them by matching against known country data. If a
    match is found, the standardized country name is included in the output list.
    Duplicates in the input list are removed in the output.

    Args:
        country_list: A list of strings, where each string represents a
            country identifier (e.g., name or code) to be cleaned and standardized.

    Returns:
        list: A list of unique, standardized country names.
    """
    washed = []
    for p in country_list:
        c = p.lower()
        r = p
        search = (
            r for r in countries if r["code"].lower() == c or r["name"].lower() == c
        )
        for s in search:
            if s is not None:
                r = s["name"]
        if r not in washed:
            washed.append(r)
    return washed


def build_country_list(self) -> list:
    """
    Builds and returns a list of countries based on the provided configuration,
    such as selected countries, timezone, or continent. The method attempts
    to determine appropriate countries for mirror usage by validating and
    processing input data, and applying fallback mechanisms when necessary.

    Returns:
        list: A list of country names determined by the provided configuration
        and mirror settings.
    """
    # Dear Fellow Manjaro Maintainer:
    # When I wrote this code, only I knew how it worked.
    # Now, no one knows!
    #
    # If you are trying to optimize
    # this function it fails (most surely),
    # please increase this counter as a warning for the next maintainer.
    #
    # total_hours_wasted_here = 1
    result = []
    if self.selected_countries:
        # this is passed in from the -c/--country commandline argument
        if self.selected_countries == ["all"]:
            result = self.mirrors.country_pool
        else:
            self.selected_countries = wash_country_list(self.selected_countries)
            if country_list_is_valid(
                onlycountry=self.selected_countries,
                countrylist=self.mirrors.country_pool,
                tty=self.tty,
            ):
                result = self.selected_countries

    # this executes when timezone or continent is True
    if not result:
        # get country
        if self.use_timezone:
            country = get_mirror_country_from_zone(get_timezone())
            if country:
                result.append(country)
            else:
                # fallback to continent
                self.continent = True

        # get a country list for the continent
        if self.continent:
            continent_mirror_countries = get_continent_countries(
                get_timezone_continent(get_timezone()), self.mirrors.mirror_pool
            )
            if continent_mirror_countries:
                result = continent_mirror_countries

        # validate the result
        # but do not return an empty list
        if not result:
            # Verify if we can fall back to using CDN.
            # One or more CDN 'Global' mirrors can be found in the mirror pool
            cdn = [x for x in self.mirrors.country_pool if x == "Global"]
            if cdn:
                result = cdn
            else:
                result = self.mirrors.country_pool

    # return the final result
    return result


def get_continent(country: str) -> str:
    """
    Database query for the continent where a country belongs
    :param country:
    """
    continent = (x for x in countries if country in x["name"])
    for x in continent:
        return x["continent"].strip().replace(" ", "_")
    return ""


def get_continent_countries(continent: str, mirror_pool: list) -> list:
    """
    Database query for countries on continent
    :param continent:
    :param mirror_pool:
    :return: list
    """
    result = []
    mirror_countries = [c for c in mirror_pool if continent in c["continent"]]
    for m in mirror_countries:
        if m["country"] not in result:
            result.append(m["country"])
    return result


def get_mirror_country_from_zone(zone: str) -> str:
    """
    Retrieves the corresponding mirror country for a given timezone.

    This function determines the country that corresponds to a specific timezone
    (zone) by examining a list of country data and their associated timezones.
    It uses data from a configuration file to map the timezone to a mirror
    country, if a match is found.

    Parameters:
    zone : str
        The timezone for which the corresponding mirror country is to be found.

    Returns:
    str
        The name of the mirror country corresponding to the specified timezone.
        Returns an empty string if no matching country is found.
    """
    import json
    from pacman_mirrors.functions.jsonFn import read_file_content
    from pacman_mirrors.config import configuration as config

    mirror_countries = json.loads(read_file_content(config.MIRROR_FILE, binary=False))
    country_zones = [x for x in countries if zone in x["timezones"]]
    if not country_zones:
        return ""
    filtered_countries = [
        x for x in mirror_countries if country_zones[0]["name"] in x["country"]
    ]
    return filtered_countries[0]["country"] if filtered_countries else ""


def get_timezone() -> str:
    """
    Provides a method to retrieve the system's current timezone.

    The function extracts the timezone information by reading the symbolic
    link at "/etc/localtime" and parsing its path. The resulting timezone
    string follows the format "<Region>/<City>".

    Returns:
        str: The current timezone in the format "<Region>/<City>".
    """
    localtime = os.readlink("/etc/localtime").split("/")
    return "{}/{}".format(localtime[-2], localtime[-1])


def get_timezone_country(timezone: str) -> str:
    """
    Fetches the country name corresponding to the given timezone.

    Given a timezone, this function matches it against a predefined list of
    countries and their associated timezones to find the country name. If no
    match is found, it returns an empty string.

    Args:
        timezone (str): The timezone string to find the corresponding country for.

    Returns:
        str: The name of the country corresponding to the given timezone.
             Returns an empty string if no match is found.
    """
    filtered_zones = [x for x in countries if timezone in x["timezones"]]
    return filtered_zones[0]["name"] if filtered_zones else ""


def get_timezone_continent(timezone: str) -> str:
    """
    Determines the continent of the current timezone.

    This function identifies the continent based on the current
    timezone's country. It performs this by comparing the current
    country against a list of countries and their corresponding
    continents.

    Returns:
        str: The continent of the current timezone, formatted with
        underscores replacing spaces. Returns an empty string if
        the continent cannot be determined.
    """
    country = get_timezone_country(timezone)
    continent = (x for x in countries if country in x["name"])
    for x in continent:
        return x["continent"].strip().replace(" ", "_")
    return ""



#!/usr/bin/env python
#
# This file is part of pacman-mirrors.
#
# pacman-mirrors is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# pacman-mirrors is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with pacman-mirrors.  If not, see <http://www.gnu.org/licenses/>.
#
# Authors: Frede Hundewadt <echo ZmhAbWFuamFyby5vcmcK | base64 -d>

"""Pacman-Mirrors Default Pool/Mirror Functions"""

from operator import itemgetter
from pacman_mirrors.functions import pools
from pacman_mirrors.functions import customFn
from pacman_mirrors.functions import fileFn


def load_config_mirror_pool(self) -> None:
    if self.use_timezone or self.continent:
        customFn.delete_custom_pool(self)
    if customFn.check_custom_pool(self) and not self.config["country_pool"]:
        customFn.load_custom_pool(self)
        self.selected_countries = self.mirrors.country_pool
    else:
        if self.config["country_pool"]:
            self.selected_countries = self.config["country_pool"]

        load_default_mirror_pool(self)
    # Validate the list of selected countries
    self.selected_countries = pools.build_country_list(self)


def load_default_mirror_pool(self) -> None:
    # Load all mirrors
    (file, status) = fileFn.return_mirror_filename(config=self.config, tty=self.tty)

    seed_mirrors(self, file, status)


def seed_mirrors(self, file: str, status: bool = False) -> None:
    mirrors = fileFn.read_mirror_file(file)
    if status:
        self.mirrors.seed(mirrors, status=status)
    else:
        self.mirrors.seed(mirrors)

    sort_mirror_countries(self)


def sort_mirror_countries(self) -> None:
    self.mirrors.mirror_pool = sorted(
        self.mirrors.mirror_pool, key=itemgetter("country")
    )

    self.mirrors.country_pool = sorted(self.mirrors.country_pool)



#!/usr/bin/env python
#
# This file is part of pacman-mirrors.
#
# pacman-mirrors is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# pacman-mirrors is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with pacman-mirrors.  If not, see <http://www.gnu.org/licenses/>.
#
# Authors: Frede Hundewadt <echo ZmhAbWFuamFyby5vcmcK | base64 -d>

"""Pacman-Mirrors Text Module"""

from pacman_mirrors.translation import i18n

_ = i18n.language.gettext

# TRANSLATABLE STRINGS

# MESSAGE TYPES
ERROR = _("ERROR")
INFO = _("INFO")
WARN = _("WARNING")
# ERROR TYPES
HTTP_EXCEPTION = _("HTTPException")
TIMEOUT = _("TIMEOUT")
# SPECIAL WORDS
API = _("API")
BRANCH = _("BRANCH")
COUNTRY = _("COUNTRY")
EXPECTED = _("EXPECTED")
FILE = _("FILE")
METHOD = _("METHOD")
METHODS = _("METHODS")
MISC = _("MISC")
NUMBER = _("NUMBER")
PATH = _("PATH")
PREFIX = _("PREFIX")
PROTO = _("PROTO")
SECONDS = _("SECONDS")
URL = _("URL")
USAGE = _("USAGE")
# HELP MESSAGES
HLP_ARG_API_GET_BRANCH = _("Return branch from configuration")
HLP_ARG_API_PREFIX = _("Set prefix to")
HLP_ARG_API_PROTOCOLS = _("Replace protocols in configuration")
HLP_ARG_API_RE_BRANCH = _("Replace branch in mirrorlist")
HLP_ARG_API_SET_BRANCH = _("Replace branch in configuration")
HLP_ARG_API_URL = _("Replace mirror url in mirrorlist")
HLP_ARG_BRANCH = _("Branch name")
HLP_ARG_COUNTRY = _(
    "Comma separated list of countries, from which mirrors will be used"
)
HLP_ARG_CONTINENT = _("Use continent from geolocation")
HLP_ARG_DEFAULT = _("Load default mirror file")
HLP_ARG_FASTTRACK = _(
    "Generate mirrorlist with a number of up-to-date mirrors. Overrides"
)
HLP_ARG_GENERATE = _("Generate mirrorlist with defaults")
HLP_ARG_TIMEZONE = _("Get mirror country using system timezone")
HLP_ARG_LIST = _("List all available countries")
HLP_ARG_INTERACTIVE = _("Generate custom mirrorlist")
HLP_ARG_INTERVAL = _("Max. number of hours since last sync")
HLP_ARG_METHOD = _("Generation method")
HLP_ARG_NO_MIRRORLIST = _("Use to skip generation of mirrorlist")
HLP_ARG_QUIET = _("Quiet mode - less verbose output")
HLP_ARG_STATUS = _("Ignore mirror branch status")
HLP_ARG_SYNC = _("Syncronize pacman databases")
HLP_ARG_TIMEOUT = _("Maximum waiting time for server response")
HLP_ARG_VERSION = _("Print the pacman-mirrors version")
HLP_ARG_MIRROR_STATUS = _("Status for the current mirror list.")
HLP_ARG_LIST_COUNTRIES = _("Lists configured mirror countries")
# GENERAL MESSAGES
API_CONF_RE_BRANCH = _("Branch in config is changed")
API_CONF_PROTOCOLS = _("Protocols in config is changed")
API_ERROR_BRANCH = _("Re-branch requires a branch to work with")
API_MIRRORLIST_RE_BRANCH = _("Branch in mirror list is changed")
AVAILABLE_COUNTRIES = _("Available countries are")
CANNOT_DOWNLOAD_FILE = _("Could not download from")
CANNOT_READ_FILE = _("Cannot read file")
CANNOT_WRITE_FILE = _("Cannot write file")
CONVERT_CUSTOM_MIRROR_FILE = _("Converting custom mirror file to new format")
CUSTOM_MIRROR_FILE = _("Custom mirror file")
CUSTOM_MIRROR_FILE_SAVED = _("Custom mirror file saved")
CUSTOM_MIRROR_LIST = _("User generated mirror list")
CUSTOM_POOL_EMPTY = _("The custom pool is empty")
DEFAULT_POOL_EMPTY = _("The default pool is empty")
DEPRECATED_ARGUMENT = _("Deprecated argument")
DOES_NOT_EXIST = _("doesn't exist.")
DOWNLOADING_MIRROR_FILE = _("Downloading mirrors from")
FALLING_BACK = _("Falling back to")
INTERNET_DOWN = _("Internet connection appears to be down")
INTERNET_ALTERNATIVE = _("Mirror list is generated using random method")
INVALID_SETTING_IN = _("Invalid setting in")
IS_MISSING = _("is missing")
MIRROR_FILE = _("The mirror file")
MIRROR_LIST_SAVED = _("Mirror list generated and saved to")
MIRROR_LIST_GENERATED_ON = _("Generated on")
MIRROR_LIST_CUSTOM_HEADER = _("custom mirrorlist")
MIRROR_LIST_CUSTOM_RESET = _("To reset custom mirrorlist")
MIRROR_LIST_DEFAULT_HEADER = _("default mirrorlist")
MIRROR_LIST_DEFAULT_MODIFY = _("to modify mirrorlist")
MIRROR_POOL_EMPTY = _("The mirror pool is empty")
MIRROR_RANKING_NA = _("Mirror ranking is not available")
MISSING_ARGUMENT = _("argument is missing")
MUST_BE_ROOT = _("Must have root privileges")
NO_CHANGE = _("The mirrors has not changed")
NO_SELECTION = _("No mirrors in selection")
OPTION = _("Option")
OVERRIDE_STATUS_CHOICE = _(
    "You have chosen to override the status for your system branch."
)
OVERRIDE_STATUS_MIRROR = _("Servers in the mirror list might not be up-to-date.")
PLEASE_USE = _("Please use")
QUERY_MIRRORS = _("Querying mirrors")
RANDOMIZING_SERVERS = _("Randomizing mirror list")
REMOVE_CUSTOM_CONFIG = _("To remove custom config run ")
TAKES_TIME = _("This may take some time")
UNKNOWN_COUNTRY = _("unknown country")
USE_ZERO_FOR_ALL = _("Use 0 for all mirrors")
USING_ALL_MIRRORS = _("Using all mirrors")
USING_CUSTOM_FILE = _("Using custom mirror file")
USING_DEFAULT_FILE = _("Using default mirror file")
WRITING_MIRROR_LIST = _("Writing mirror list")

# INTERACTIVE MESSAGES
I_TITLE = _("Manjaro mirrors by response time")
I_TITLE_RANDOM = _("Manjaro mirrors in random order")
I_LIST_TITLE = _("Check mirrors for your personal list")
I_USE = _("Use")
I_COUNTRY = _("Country")
I_RESPONSE = _("Resp")
I_LAST_SYNC = _("Sync")
I_URL = _("URL")
I_CANCEL = _("Cancel")
I_CONFIRM = _("OK")
I_CONFIRM_SELECTION = _("Confirm selections")
I_USE_THESE_MIRRORS = _("I want to use these mirrors")
# NON TRANSLATABLE STRINGS
ARGUMENT_API = "-a/--api"
ARGUMENT_DEFAULT = "-d/--default"
ARGUMENT_INTERACTIVE = "-i/--interactive"
HOUSTON = "Houston?! We have a problem."
OVERRIDE_OPT = ": --timezone, --method"
REPO_SERVER = "repo.manjaro.org"
MODIFY_CUSTOM = "pacman-mirrors -id"
MODIFY_DEFAULT = f"pacman-mirrors -f [{NUMBER}]"
RESET_ALL = "pacman-mirrors -c all"
PREFIX_TIP = ": $mnt | /mnt/install"
# OPTIONS
OPT_RANDOM = " '-m/--method random' "
OPT_COUNTRY = " 'c/--country' "
# MIRROR RELATED
LASTSYNC_OK = "24:00"  # last syncronize in the past 24 hours
LASTSYNC_NA = "9800:00"  # last syncronization not available
SERVER_BAD = "9999:99"  # default last syncronization status
SERVER_RES = 99.99  # default response status
# MESSAGE WITH COLOR
ERR_CLR = f"\033[1;31m{ERROR}\033[1;m"
INF_CLR = f"\033[1;37m{INFO}\033[1;m"
WRN_CLR = f"\033[1;33m{WARN}\033[1;m"
EXP_CLR = f"\033[1;33m{EXPECTED}\033[1;m"
API_ARGUMENTS_ERROR = f"{ARGUMENT_API} {MISSING_ARGUMENT}."
INTERACTIVE_ARGUMENTS_ERROR = f"{ARGUMENT_INTERACTIVE} {MISSING_ARGUMENT}."
FASTTRACK_ARGUMENTS_ERROR = (
    "pacman-mirrors: error: argument -s/--no-status: "
    "not allowed with argument -f/--fasttrack"
)
ASYNC_DISCLAIM = [
    f"{WRN_CLR}",
    f"{WRN_CLR} THE ASYNC ARGUMENT IS EXPERIMENTAL",
    f"{WRN_CLR} PROCESS MAY HANG - USE CTRL+C OR CTRL+D",
    "REPORT URL: https://gitlab.manjaro.org/applications/pacman-mirrors/issues/",
    f"{WRN_CLR} Acknowledge and continue? (Y/n) \r",
]
